# -*- coding: utf-8 -*-
# Generated by Django 1.9.10 on 2016-11-25 11:01
from __future__ import unicode_literals

import cms.models.fields
from decimal import Decimal
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import django_fsm
import django_measurement.models
import filer.fields.file
import filer.fields.image
import mptt.fields
import parler.models
import shop.models.customer
import shop.models.fields
import shop.money.fields
import shop.payment.defaults


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '__first__'),
        ('filer', '__first__'),
        ('contenttypes', '__first__'),
        ('email_auth', '__first__'),
    ]

    operations = [
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('kind', models.CharField(choices=[('image', 'Image'), ('video', 'Video'), ('file', 'File')], default='image', max_length=16, verbose_name='Kind')),
                ('url', models.URLField(blank=True, verbose_name='Url')),
                ('order', models.PositiveIntegerField(default=0, verbose_name='Sort')),
                ('file', filer.fields.file.FilerFileField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='filer.File', verbose_name='File')),
            ],
            options={
                'ordering': ['order'],
                'db_table': 'shopit_attachments',
                'verbose_name': 'Attachment',
                'verbose_name_plural': 'Attachments',
            },
        ),
        migrations.CreateModel(
            name='Attribute',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.SlugField(help_text="An identifier that's used to access this attribute. Must be unique.", max_length=128, unique=True, verbose_name='Code')),
                ('template', models.CharField(blank=True, help_text='You can specify a template for rendering this attribute or leave it empty for the default (dropdown) look.', max_length=255, null=True, verbose_name='Template')),
                ('nullable', models.BooleanField(default=False, help_text='Check this if you want to make "empty" an option for this Attribute.', verbose_name='Nullable')),
                ('active', models.BooleanField(default=True, help_text='Is this attribute publicly visible.', verbose_name='Active')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('order', models.PositiveIntegerField(default=0, verbose_name='Sort')),
            ],
            options={
                'ordering': ['order'],
                'db_table': 'shopit_attributes',
                'verbose_name': 'Attribute',
                'verbose_name_plural': 'Attributes',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='AttributeChoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.CharField(max_length=255, verbose_name='Value')),
                ('order', models.PositiveIntegerField(default=0, verbose_name='Sort')),
                ('attribute', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='choices', to='shopit.Attribute', verbose_name='Attribute')),
                ('file', filer.fields.file.FilerFileField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='filer.File', verbose_name='File')),
            ],
            options={
                'ordering': ['order'],
                'db_table': 'shopit_attribute_choices',
                'verbose_name': 'Choice',
                'verbose_name_plural': 'Choices',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='AttributeChoiceTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('name', models.CharField(blank=True, max_length=128, verbose_name='Name')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='shopit.AttributeChoice')),
            ],
            options={
                'managed': True,
                'db_table': 'shopit_attribute_choices_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'Choice Translation',
            },
        ),
        migrations.CreateModel(
            name='AttributeTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='shopit.Attribute')),
            ],
            options={
                'managed': True,
                'db_table': 'shopit_attributes_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'Attribute Translation',
            },
        ),
        migrations.CreateModel(
            name='AttributeValue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('attribute', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='values', to='shopit.Attribute', verbose_name='Attribute')),
                ('choice', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='shopit.AttributeChoice', verbose_name='Value')),
            ],
            options={
                'ordering': ['attribute__order'],
                'db_table': 'shopit_attribute_values',
                'verbose_name': 'Attribute Value',
                'verbose_name_plural': 'Attribute Values',
            },
        ),
        migrations.CreateModel(
            name='BillingAddress',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('priority', models.SmallIntegerField(help_text='Priority for using this address')),
                ('name', models.CharField(max_length=1024, verbose_name='Full name')),
                ('address1', models.CharField(max_length=1024, verbose_name='Address line 1')),
                ('address2', models.CharField(blank=True, max_length=1024, null=True, verbose_name='Address line 2')),
                ('zip_code', models.CharField(max_length=12, verbose_name='ZIP / Postal code')),
                ('city', models.CharField(max_length=1024, verbose_name='City')),
                ('country', models.CharField(choices=[('AF', 'Afghanistan'), ('AX', 'Aland Islands'), ('AL', 'Albania'), ('DZ', 'Algeria'), ('AS', 'American Samoa'), ('AD', 'Andorra'), ('AO', 'Angola'), ('AI', 'Anguilla'), ('AQ', 'Antarctica'), ('AG', 'Antigua And Barbuda'), ('AR', 'Argentina'), ('AM', 'Armenia'), ('AW', 'Aruba'), ('AU', 'Australia'), ('AT', 'Austria'), ('AZ', 'Azerbaijan'), ('BS', 'Bahamas'), ('BH', 'Bahrain'), ('BD', 'Bangladesh'), ('BB', 'Barbados'), ('BY', 'Belarus'), ('BE', 'Belgium'), ('BZ', 'Belize'), ('BJ', 'Benin'), ('BM', 'Bermuda'), ('BT', 'Bhutan'), ('BO', 'Bolivia, Plurinational State Of'), ('BQ', 'Bonaire, Saint Eustatius And Saba'), ('BA', 'Bosnia And Herzegovina'), ('BW', 'Botswana'), ('BV', 'Bouvet Island'), ('BR', 'Brazil'), ('IO', 'British Indian Ocean Territory'), ('BN', 'Brunei Darussalam'), ('BG', 'Bulgaria'), ('BF', 'Burkina Faso'), ('BI', 'Burundi'), ('KH', 'Cambodia'), ('CM', 'Cameroon'), ('CA', 'Canada'), ('CV', 'Cape Verde'), ('KY', 'Cayman Islands'), ('CF', 'Central African Republic'), ('TD', 'Chad'), ('CL', 'Chile'), ('CN', 'China'), ('CX', 'Christmas Island'), ('CC', 'Cocos (Keeling) Islands'), ('CO', 'Colombia'), ('KM', 'Comoros'), ('CG', 'Congo'), ('CD', 'Congo, The Democratic Republic Of The'), ('CK', 'Cook Islands'), ('CR', 'Costa Rica'), ('HR', 'Croatia'), ('CU', 'Cuba'), ('CW', 'Curacao'), ('CY', 'Cyprus'), ('CZ', 'Czech Republic'), ('DK', 'Denmark'), ('DJ', 'Djibouti'), ('DM', 'Dominica'), ('DO', 'Dominican Republic'), ('EC', 'Ecuador'), ('EG', 'Egypt'), ('SV', 'El Salvador'), ('GQ', 'Equatorial Guinea'), ('ER', 'Eritrea'), ('EE', 'Estonia'), ('ET', 'Ethiopia'), ('FK', 'Falkland Islands (Malvinas)'), ('FO', 'Faroe Islands'), ('FJ', 'Fiji'), ('FI', 'Finland'), ('FR', 'France'), ('GF', 'French Guiana'), ('PF', 'French Polynesia'), ('TF', 'French Southern Territories'), ('GA', 'Gabon'), ('GM', 'Gambia'), ('DE', 'Germany'), ('GH', 'Ghana'), ('GI', 'Gibraltar'), ('GR', 'Greece'), ('GL', 'Greenland'), ('GD', 'Grenada'), ('GP', 'Guadeloupe'), ('GU', 'Guam'), ('GT', 'Guatemala'), ('GG', 'Guernsey'), ('GN', 'Guinea'), ('GW', 'Guinea-Bissau'), ('GY', 'Guyana'), ('HT', 'Haiti'), ('HM', 'Heard Island and McDonald Islands'), ('VA', 'Holy See (Vatican City State)'), ('HN', 'Honduras'), ('HK', 'Hong Kong'), ('HU', 'Hungary'), ('IS', 'Iceland'), ('IN', 'India'), ('ID', 'Indonesia'), ('IR', 'Iran, Islamic Republic Of'), ('IQ', 'Iraq'), ('IE', 'Ireland'), ('IL', 'Israel'), ('IT', 'Italy'), ('CI', 'Ivory Coast'), ('JM', 'Jamaica'), ('JP', 'Japan'), ('JE', 'Jersey'), ('JO', 'Jordan'), ('KZ', 'Kazakhstan'), ('KE', 'Kenya'), ('KI', 'Kiribati'), ('KP', "Korea, Democratic People's Republic Of"), ('KR', 'Korea, Republic Of'), ('KS', 'Kosovo'), ('KW', 'Kuwait'), ('KG', 'Kyrgyzstan'), ('LA', "Lao People's Democratic Republic"), ('LV', 'Latvia'), ('LB', 'Lebanon'), ('LS', 'Lesotho'), ('LR', 'Liberia'), ('LY', 'Libyan Arab Jamahiriya'), ('LI', 'Liechtenstein'), ('LT', 'Lithuania'), ('LU', 'Luxembourg'), ('MO', 'Macao'), ('MK', 'Macedonia'), ('MG', 'Madagascar'), ('MW', 'Malawi'), ('MY', 'Malaysia'), ('MV', 'Maldives'), ('ML', 'Mali'), ('ML', 'Malta'), ('MH', 'Marshall Islands'), ('MQ', 'Martinique'), ('MR', 'Mauritania'), ('MU', 'Mauritius'), ('YT', 'Mayotte'), ('MX', 'Mexico'), ('FM', 'Micronesia'), ('MD', 'Moldova'), ('MC', 'Monaco'), ('MN', 'Mongolia'), ('ME', 'Montenegro'), ('MS', 'Montserrat'), ('MA', 'Morocco'), ('MZ', 'Mozambique'), ('MM', 'Myanmar'), ('NA', 'Namibia'), ('NR', 'Nauru'), ('NP', 'Nepal'), ('NL', 'Netherlands'), ('AN', 'Netherlands Antilles'), ('NC', 'New Caledonia'), ('NZ', 'New Zealand'), ('NI', 'Nicaragua'), ('NE', 'Niger'), ('NG', 'Nigeria'), ('NU', 'Niue'), ('NF', 'Norfolk Island'), ('MP', 'Northern Mariana Islands'), ('NO', 'Norway'), ('OM', 'Oman'), ('PK', 'Pakistan'), ('PW', 'Palau'), ('PS', 'Palestinian Territory, Occupied'), ('PA', 'Panama'), ('PG', 'Papua New Guinea'), ('PY', 'Paraguay'), ('PE', 'Peru'), ('PH', 'Philippines'), ('PN', 'Pitcairn'), ('PL', 'Poland'), ('PT', 'Portugal'), ('PR', 'Puerto Rico'), ('QA', 'Qatar'), ('RE', 'Reunion'), ('RO', 'Romania'), ('RU', 'Russian Federation'), ('RW', 'Rwanda'), ('BL', 'Saint Barthelemy'), ('SH', 'Saint Helena, Ascension & Tristan Da Cunha'), ('KN', 'Saint Kitts and Nevis'), ('LC', 'Saint Lucia'), ('MF', 'Saint Martin (French Part)'), ('PM', 'Saint Pierre and Miquelon'), ('VC', 'Saint Vincent And The Grenadines'), ('WS', 'Samoa'), ('SM', 'San Marino'), ('ST', 'Sao Tome And Principe'), ('SA', 'Saudi Arabia'), ('SN', 'Senegal'), ('RS', 'Serbia'), ('SC', 'Seychelles'), ('SL', 'Sierra Leone'), ('SG', 'Singapore'), ('SX', 'Sint Maarten (Dutch Part)'), ('SK', 'Slovakia'), ('SI', 'Slovenia'), ('SB', 'Solomon Islands'), ('SO', 'Somalia'), ('ZA', 'South Africa'), ('GS', 'South Georgia And The South Sandwich Islands'), ('ES', 'Spain'), ('LK', 'Sri Lanka'), ('SD', 'Sudan'), ('SR', 'Suriname'), ('SJ', 'Svalbard And Jan Mayen'), ('SZ', 'Swaziland'), ('SE', 'Sweden'), ('CH', 'Switzerland'), ('SY', 'Syrian Arab Republic'), ('TW', 'Taiwan'), ('TJ', 'Tajikistan'), ('TZ', 'Tanzania'), ('TH', 'Thailand'), ('TL', 'Timor-Leste'), ('TG', 'Togo'), ('TK', 'Tokelau'), ('TO', 'Tonga'), ('TT', 'Trinidad and Tobago'), ('TN', 'Tunisia'), ('TR', 'Turkey'), ('TM', 'Turkmenistan'), ('TC', 'Turks And Caicos Islands'), ('TV', 'Tuvalu'), ('UG', 'Uganda'), ('UA', 'Ukraine'), ('AE', 'United Arab Emirates'), ('GB', 'United Kingdom'), ('US', 'United States'), ('UM', 'United States Minor Outlying Islands'), ('UY', 'Uruguay'), ('UZ', 'Uzbekistan'), ('VU', 'Vanuatu'), ('VE', 'Venezuela, Bolivarian Republic Of'), ('VN', 'Viet Nam'), ('VG', 'Virgin Islands, British'), ('VI', 'Virgin Islands, U.S.'), ('WF', 'Wallis and Futuna'), ('EH', 'Western Sahara'), ('YE', 'Yemen'), ('ZM', 'Zambia'), ('ZW', 'Zimbabwe')], max_length=3, verbose_name='Country')),
            ],
            options={
                'db_table': 'shopit_billing_addresses',
                'verbose_name': 'Billing address',
                'verbose_name_plural': 'Billing addresses',
            },
        ),
        migrations.CreateModel(
            name='Brand',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('active', models.BooleanField(default=True, help_text='Is this categorization publicly visible.', verbose_name='Active')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('lft', models.PositiveIntegerField(db_index=True, editable=False)),
                ('rght', models.PositiveIntegerField(db_index=True, editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(db_index=True, editable=False)),
                ('_featured_image', filer.fields.image.FilerImageField(blank=True, help_text="If left empty for childs, a parent's featured image will be used.", null=True, on_delete=django.db.models.deletion.SET_NULL, to='filer.Image', verbose_name='Featured image')),
            ],
            options={
                'ordering': ['tree_id', 'lft'],
                'db_table': 'shopit_brands',
                'verbose_name': 'Brand',
                'verbose_name_plural': 'Brands',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='BrandTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('slug', models.SlugField(verbose_name='Slug')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='shopit.Brand')),
            ],
            options={
                'managed': True,
                'db_table': 'shopit_brands_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'Brand Translation',
            },
        ),
        migrations.CreateModel(
            name='Cart',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('extra', shop.models.fields.JSONField(verbose_name='Arbitrary information for this cart')),
                ('billing_address', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='shopit.BillingAddress')),
            ],
            options={
                'db_table': 'shopit_carts',
                'verbose_name': 'Shopping cart',
                'verbose_name_plural': 'Shopping carts',
            },
        ),
        migrations.CreateModel(
            name='CartDiscountCode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=30, verbose_name='Code')),
                ('cart', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='discount_codes', to='shopit.Cart')),
            ],
            options={
                'db_table': 'shopit_cart_discount_codes',
                'verbose_name': 'Cart discount code',
                'verbose_name_plural': 'Cart discount codes',
            },
        ),
        migrations.CreateModel(
            name='CartItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('extra', shop.models.fields.JSONField(verbose_name='Arbitrary information for this cart item')),
                ('quantity', models.IntegerField(validators=[django.core.validators.MinValueValidator(0)])),
                ('cart', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='items', to='shopit.Cart')),
            ],
            options={
                'db_table': 'shopit_cart_items',
                'verbose_name': 'Cart item',
                'verbose_name_plural': 'Cart items',
            },
        ),
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('active', models.BooleanField(default=True, help_text='Is this categorization publicly visible.', verbose_name='Active')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('lft', models.PositiveIntegerField(db_index=True, editable=False)),
                ('rght', models.PositiveIntegerField(db_index=True, editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(db_index=True, editable=False)),
                ('_featured_image', filer.fields.image.FilerImageField(blank=True, help_text="If left empty for childs, a parent's featured image will be used.", null=True, on_delete=django.db.models.deletion.SET_NULL, to='filer.Image', verbose_name='Featured image')),
            ],
            options={
                'ordering': ['tree_id', 'lft'],
                'db_table': 'shopit_categories',
                'verbose_name': 'Category',
                'verbose_name_plural': 'Categories',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='CategoryTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('slug', models.SlugField(verbose_name='Slug')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='shopit.Category')),
            ],
            options={
                'managed': True,
                'db_table': 'shopit_categories_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'Category Translation',
            },
        ),
        migrations.CreateModel(
            name='Customer',
            fields=[
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, serialize=False, to=settings.AUTH_USER_MODEL)),
                ('recognized', shop.models.fields.ChoiceEnumField(help_text='Designates the state the customer is recognized as.', verbose_name='Recognized as')),
                ('salutation', models.CharField(choices=[('mrs', 'Mrs.'), ('mr', 'Mr.'), ('na', '(n/a)')], max_length=5, verbose_name='Salutation')),
                ('last_access', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Last accessed')),
                ('extra', shop.models.fields.JSONField(editable=False, verbose_name='Extra information about this customer')),
                ('number', models.PositiveIntegerField(default=None, null=True, unique=True, verbose_name='Customer Number')),
            ],
            options={
                'db_table': 'shopit_customers',
                'verbose_name': 'Customer',
                'verbose_name_plural': 'Customers',
            },
        ),
        migrations.CreateModel(
            name='Delivery',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('shipping_id', models.CharField(blank=True, help_text="The transaction processor's reference", max_length=255, null=True, verbose_name='Shipping ID')),
                ('fulfilled_at', models.DateTimeField(blank=True, null=True, verbose_name='Fulfilled at')),
                ('shipped_at', models.DateTimeField(blank=True, null=True, verbose_name='Shipped at')),
                ('shipping_method', models.CharField(help_text='The shipping backend used to deliver the items for this order', max_length=50, verbose_name='Shipping method')),
            ],
            options={
                'db_table': 'shopit_deliveries',
                'verbose_name': 'Delivery',
                'verbose_name_plural': 'Deliveries',
            },
        ),
        migrations.CreateModel(
            name='DeliveryItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('quantity', models.IntegerField(default=0, verbose_name='Delivered quantity')),
                ('delivery', models.ForeignKey(help_text='Refer to the shipping provider used to ship this item', on_delete=django.db.models.deletion.CASCADE, to='shopit.Delivery', verbose_name='Delivery')),
            ],
            options={
                'db_table': 'shopit_delivery_items',
                'verbose_name': 'Delivery item',
                'verbose_name_plural': 'Delivery items',
            },
        ),
        migrations.CreateModel(
            name='DiscountCode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(help_text='Code that must be entered for the modifier to activate.', max_length=30, unique=True, verbose_name='Code')),
                ('active', models.BooleanField(default=True, verbose_name='Active')),
                ('valid_from', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Valid from')),
                ('valid_until', models.DateTimeField(blank=True, null=True, verbose_name='Valid until')),
                ('order', models.PositiveIntegerField(default=0, verbose_name='Sort')),
                ('customer', models.ForeignKey(blank=True, help_text='Limit code so that it can be used only by a specific customer.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='discount_codes', to='shopit.Customer', verbose_name='Customer')),
            ],
            options={
                'ordering': ['order'],
                'db_table': 'shopit_discount_codes',
                'verbose_name': 'Discount code',
                'verbose_name_plural': 'Discount codes',
            },
        ),
        migrations.CreateModel(
            name='Flag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.SlugField(help_text='Unique identifier for this flag.', unique=True, verbose_name='Code')),
                ('active', models.BooleanField(default=True, help_text='Is this flag publicly visible.', verbose_name='Active')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('order', models.PositiveIntegerField(default=0, verbose_name='Sort')),
            ],
            options={
                'ordering': ['order'],
                'db_table': 'shopit_flags',
                'verbose_name': 'Flag',
                'verbose_name_plural': 'Flags',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='FlagTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='shopit.Flag')),
            ],
            options={
                'managed': True,
                'db_table': 'shopit_flags_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'Flag Translation',
            },
        ),
        migrations.CreateModel(
            name='Manufacturer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('active', models.BooleanField(default=True, help_text='Is this categorization publicly visible.', verbose_name='Active')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('lft', models.PositiveIntegerField(db_index=True, editable=False)),
                ('rght', models.PositiveIntegerField(db_index=True, editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(db_index=True, editable=False)),
                ('_featured_image', filer.fields.image.FilerImageField(blank=True, help_text="If left empty for childs, a parent's featured image will be used.", null=True, on_delete=django.db.models.deletion.SET_NULL, to='filer.Image', verbose_name='Featured image')),
                ('flags', models.ManyToManyField(blank=True, help_text='Check flags for products in this categorization.', to='shopit.Flag', verbose_name='Flags')),
            ],
            options={
                'ordering': ['tree_id', 'lft'],
                'db_table': 'shopit_manufacturers',
                'verbose_name': 'Manufacturer',
                'verbose_name_plural': 'Manufacturers',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ManufacturerTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('slug', models.SlugField(verbose_name='Slug')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='shopit.Manufacturer')),
            ],
            options={
                'managed': True,
                'db_table': 'shopit_manufacturers_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'Manufacturer Translation',
            },
        ),
        migrations.CreateModel(
            name='Modifier',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.SlugField(help_text='Unique identifier for this modifier.', unique=True, verbose_name='Code')),
                ('amount', shop.money.fields.MoneyField(help_text='Amount that should be added. Can be negative.', verbose_name='Amount')),
                ('percent', models.DecimalField(blank=True, decimal_places=2, help_text='Percent that should be added, overrides the amount. Can be negative.', max_digits=4, null=True, verbose_name='Percent')),
                ('kind', models.CharField(choices=[('standard', 'Standard'), ('discount', 'Discount'), ('cart', 'Cart')], default='standard', help_text='Standard affects the product regardles, Discount checks for a "Discountable" flag on a product and should be negative, Cart will affect an entire cart.', max_length=16, verbose_name='Kind')),
                ('active', models.BooleanField(default=True, help_text='Is this modifier publicly visible.', verbose_name='Active')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('order', models.PositiveIntegerField(default=0, verbose_name='Sort')),
            ],
            options={
                'ordering': ['order'],
                'db_table': 'shopit_modifiers',
                'verbose_name': 'Modifier',
                'verbose_name_plural': 'Modifiers',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ModifierCondition',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.CharField(blank=True, choices=[('shopit.modifier_conditions.PriceGreaterThanCondition', 'Price greater than'), ('shopit.modifier_conditions.PriceLessThanCondition', 'Price less than'), ('shopit.modifier_conditions.QuantityGreaterThanCondition', 'Quantity greater than'), ('shopit.modifier_conditions.QuantityLessThanCondition', 'Quantity less than')], max_length=255, verbose_name='Condition')),
                ('value', models.DecimalField(blank=True, decimal_places=2, max_digits=10, null=True, verbose_name='Value')),
                ('order', models.PositiveIntegerField(default=0, verbose_name='Sort')),
                ('modifier', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='conditions', to='shopit.Modifier', verbose_name='Modifier')),
            ],
            options={
                'ordering': ['order'],
                'db_table': 'shopit_modifier_conditions',
                'verbose_name': 'Condition',
                'verbose_name_plural': 'Conditions',
            },
        ),
        migrations.CreateModel(
            name='ModifierTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='shopit.Modifier')),
            ],
            options={
                'managed': True,
                'db_table': 'shopit_modifiers_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'Modifier Translation',
            },
        ),
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', django_fsm.FSMField(default='new', max_length=50, protected=True, verbose_name='Status')),
                ('currency', models.CharField(editable=False, help_text='Currency in which this order was concluded', max_length=7)),
                ('_subtotal', models.DecimalField(decimal_places=2, max_digits=30, verbose_name='Subtotal')),
                ('_total', models.DecimalField(decimal_places=2, max_digits=30, verbose_name='Total')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('extra', shop.models.fields.JSONField(help_text='Arbitrary information for this order object on the moment of purchase.', verbose_name='Extra fields')),
                ('stored_request', shop.models.fields.JSONField(help_text='Parts of the Request objects on the moment of purchase.')),
                ('number', models.PositiveIntegerField(default=None, null=True, unique=True, verbose_name='Order Number')),
                ('shipping_address_text', models.TextField(blank=True, help_text='Shipping address at the moment of purchase.', null=True, verbose_name='Shipping Address')),
                ('billing_address_text', models.TextField(blank=True, help_text='Billing address at the moment of purchase.', null=True, verbose_name='Billing Address')),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='orders', to='shopit.Customer', verbose_name='Customer')),
            ],
            options={
                'db_table': 'shopit_orders',
                'verbose_name': 'Order',
                'verbose_name_plural': 'Orders',
            },
        ),
        migrations.CreateModel(
            name='OrderItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('product_name', models.CharField(blank=True, help_text='Product name at the moment of purchase.', max_length=255, null=True, verbose_name='Product name')),
                ('product_code', models.CharField(blank=True, help_text='Product code at the moment of purchase.', max_length=255, null=True, verbose_name='Product code')),
                ('_unit_price', models.DecimalField(decimal_places=2, help_text='Products unit price at the moment of purchase.', max_digits=30, null=True, verbose_name='Unit price')),
                ('_line_total', models.DecimalField(decimal_places=2, help_text='Line total on the invoice at the moment of purchase.', max_digits=30, null=True, verbose_name='Line Total')),
                ('extra', shop.models.fields.JSONField(help_text='Arbitrary information for this order item', verbose_name='Extra fields')),
                ('quantity', models.IntegerField(verbose_name='Ordered quantity')),
                ('canceled', models.BooleanField(default=False, verbose_name='Item canceled')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='items', to='shopit.Order', verbose_name='Order')),
            ],
            options={
                'db_table': 'shopit_order_items',
                'verbose_name': 'Order item',
                'verbose_name_plural': 'Order items',
            },
        ),
        migrations.CreateModel(
            name='OrderPayment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', shop.money.fields.MoneyField(help_text='How much was paid with this particular transfer.', verbose_name='Amount paid')),
                ('transaction_id', models.CharField(help_text="The transaction processor's reference", max_length=255, verbose_name='Transaction ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Received at')),
                ('payment_method', models.CharField(help_text='The payment backend used to process the purchase', max_length=50, verbose_name='Payment method')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopit.Order', verbose_name='Order')),
            ],
            options={
                'verbose_name': 'Order payment',
                'verbose_name_plural': 'Order payments',
            },
        ),
        migrations.CreateModel(
            name='Product',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('active', models.BooleanField(default=True, help_text='Is this product publicly visible.', verbose_name='Active')),
                ('code', models.CharField(help_text='Unique identifier for a product.', max_length=64, unique=True, verbose_name='Code')),
                ('_unit_price', shop.money.fields.MoneyField(blank=True, help_text='For variants leave empty to use the Group price.', null=True, verbose_name='Unit price')),
                ('_discount', models.DecimalField(blank=True, decimal_places=2, help_text='For variants leave empty to use Group discount.', max_digits=4, null=True, validators=[django.core.validators.MinValueValidator(Decimal('0.00'))], verbose_name='Discount %')),
                ('kind', models.PositiveSmallIntegerField(choices=[(0, 'Single'), (1, 'Group'), (2, 'Variant')], default=0, help_text='Choose a product type. Single products are products without variations. Group products are base products that hold variants and their common info, they cannot be added to cart. Variants are variations of a product that must select a Group product, and set their unique set of attributes. (See "Variant" section below)', verbose_name='Kind')),
                ('discountable', models.BooleanField(default=True, help_text='Can this product be used in an offer?', verbose_name='Discountable')),
                ('_width', django_measurement.models.MeasurementField(blank=True, measurement_class='Distance', null=True, verbose_name='Width')),
                ('_height', django_measurement.models.MeasurementField(blank=True, measurement_class='Distance', null=True, verbose_name='Height')),
                ('_depth', django_measurement.models.MeasurementField(blank=True, measurement_class='Distance', null=True, verbose_name='Depth')),
                ('_weight', django_measurement.models.MeasurementField(blank=True, measurement_class='Mass', null=True, verbose_name='Weight')),
                ('published', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Published')),
                ('quantity', models.IntegerField(blank=True, help_text='Number of available products to ship. Leave empty if product is always available, or set to 0 if product is not available.', null=True, verbose_name='Quantity')),
                ('order', models.BigIntegerField(default=0, verbose_name='Sort')),
                ('_brand', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='shopit.Brand', verbose_name='Brand')),
                ('_category', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='shopit.Category', verbose_name='Category')),
                ('_manufacturer', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='shopit.Manufacturer', verbose_name='Manufacturer')),
            ],
            options={
                'ordering': ['-order'],
                'db_table': 'shopit_products',
                'verbose_name': 'Product',
                'verbose_name_plural': 'Products',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ProductTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('slug', models.SlugField(help_text="Part that's used in url to display this product. Needs to be unique.", verbose_name='Slug')),
                ('caption', models.TextField(blank=True, help_text="Short product caption, usually used in catalog's list view of products.", max_length=255, verbose_name='Caption')),
                ('description', models.TextField(blank=True, help_text="Description of a product, usually used as lead text in product's detail view.", verbose_name='Description')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='shopit.Product')),
            ],
            options={
                'managed': True,
                'db_table': 'shopit_products_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'Product Translation',
            },
        ),
        migrations.CreateModel(
            name='Relation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('kind', models.CharField(blank=True, choices=[('up-sell', 'Up-sell'), ('cross-sell', 'Cross-sell')], max_length=128, verbose_name='Kind')),
                ('order', models.PositiveIntegerField(default=0, verbose_name='Sort')),
                ('base', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='relations', to='shopit.Product')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopit.Product', verbose_name='Product')),
            ],
            options={
                'ordering': ['order'],
                'db_table': 'shopit_relations',
                'verbose_name': 'Relation',
                'verbose_name_plural': 'Relations',
            },
        ),
        migrations.CreateModel(
            name='Review',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('rating', models.PositiveIntegerField(default=0, verbose_name='Rating')),
                ('active', models.BooleanField(default=True, help_text='Is this review publicly visible.', verbose_name='Active')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
                ('order', models.PositiveIntegerField(default=0, verbose_name='Sort')),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='product_reviews', to='shopit.Customer', verbose_name='Customer')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reviews', to='shopit.Product', verbose_name='Product')),
            ],
            options={
                'ordering': ['-order'],
                'db_table': 'shopit_reviews',
                'verbose_name': 'Review',
                'verbose_name_plural': 'Reviews',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ReviewTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('body', models.TextField(max_length=1024, verbose_name='Body')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='shopit.Review')),
            ],
            options={
                'managed': True,
                'db_table': 'shopit_reviews_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'Review Translation',
            },
        ),
        migrations.CreateModel(
            name='ShippingAddress',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('priority', models.SmallIntegerField(help_text='Priority for using this address')),
                ('name', models.CharField(max_length=1024, verbose_name='Full name')),
                ('address1', models.CharField(max_length=1024, verbose_name='Address line 1')),
                ('address2', models.CharField(blank=True, max_length=1024, null=True, verbose_name='Address line 2')),
                ('zip_code', models.CharField(max_length=12, verbose_name='ZIP / Postal code')),
                ('city', models.CharField(max_length=1024, verbose_name='City')),
                ('country', models.CharField(choices=[('AF', 'Afghanistan'), ('AX', 'Aland Islands'), ('AL', 'Albania'), ('DZ', 'Algeria'), ('AS', 'American Samoa'), ('AD', 'Andorra'), ('AO', 'Angola'), ('AI', 'Anguilla'), ('AQ', 'Antarctica'), ('AG', 'Antigua And Barbuda'), ('AR', 'Argentina'), ('AM', 'Armenia'), ('AW', 'Aruba'), ('AU', 'Australia'), ('AT', 'Austria'), ('AZ', 'Azerbaijan'), ('BS', 'Bahamas'), ('BH', 'Bahrain'), ('BD', 'Bangladesh'), ('BB', 'Barbados'), ('BY', 'Belarus'), ('BE', 'Belgium'), ('BZ', 'Belize'), ('BJ', 'Benin'), ('BM', 'Bermuda'), ('BT', 'Bhutan'), ('BO', 'Bolivia, Plurinational State Of'), ('BQ', 'Bonaire, Saint Eustatius And Saba'), ('BA', 'Bosnia And Herzegovina'), ('BW', 'Botswana'), ('BV', 'Bouvet Island'), ('BR', 'Brazil'), ('IO', 'British Indian Ocean Territory'), ('BN', 'Brunei Darussalam'), ('BG', 'Bulgaria'), ('BF', 'Burkina Faso'), ('BI', 'Burundi'), ('KH', 'Cambodia'), ('CM', 'Cameroon'), ('CA', 'Canada'), ('CV', 'Cape Verde'), ('KY', 'Cayman Islands'), ('CF', 'Central African Republic'), ('TD', 'Chad'), ('CL', 'Chile'), ('CN', 'China'), ('CX', 'Christmas Island'), ('CC', 'Cocos (Keeling) Islands'), ('CO', 'Colombia'), ('KM', 'Comoros'), ('CG', 'Congo'), ('CD', 'Congo, The Democratic Republic Of The'), ('CK', 'Cook Islands'), ('CR', 'Costa Rica'), ('HR', 'Croatia'), ('CU', 'Cuba'), ('CW', 'Curacao'), ('CY', 'Cyprus'), ('CZ', 'Czech Republic'), ('DK', 'Denmark'), ('DJ', 'Djibouti'), ('DM', 'Dominica'), ('DO', 'Dominican Republic'), ('EC', 'Ecuador'), ('EG', 'Egypt'), ('SV', 'El Salvador'), ('GQ', 'Equatorial Guinea'), ('ER', 'Eritrea'), ('EE', 'Estonia'), ('ET', 'Ethiopia'), ('FK', 'Falkland Islands (Malvinas)'), ('FO', 'Faroe Islands'), ('FJ', 'Fiji'), ('FI', 'Finland'), ('FR', 'France'), ('GF', 'French Guiana'), ('PF', 'French Polynesia'), ('TF', 'French Southern Territories'), ('GA', 'Gabon'), ('GM', 'Gambia'), ('DE', 'Germany'), ('GH', 'Ghana'), ('GI', 'Gibraltar'), ('GR', 'Greece'), ('GL', 'Greenland'), ('GD', 'Grenada'), ('GP', 'Guadeloupe'), ('GU', 'Guam'), ('GT', 'Guatemala'), ('GG', 'Guernsey'), ('GN', 'Guinea'), ('GW', 'Guinea-Bissau'), ('GY', 'Guyana'), ('HT', 'Haiti'), ('HM', 'Heard Island and McDonald Islands'), ('VA', 'Holy See (Vatican City State)'), ('HN', 'Honduras'), ('HK', 'Hong Kong'), ('HU', 'Hungary'), ('IS', 'Iceland'), ('IN', 'India'), ('ID', 'Indonesia'), ('IR', 'Iran, Islamic Republic Of'), ('IQ', 'Iraq'), ('IE', 'Ireland'), ('IL', 'Israel'), ('IT', 'Italy'), ('CI', 'Ivory Coast'), ('JM', 'Jamaica'), ('JP', 'Japan'), ('JE', 'Jersey'), ('JO', 'Jordan'), ('KZ', 'Kazakhstan'), ('KE', 'Kenya'), ('KI', 'Kiribati'), ('KP', "Korea, Democratic People's Republic Of"), ('KR', 'Korea, Republic Of'), ('KS', 'Kosovo'), ('KW', 'Kuwait'), ('KG', 'Kyrgyzstan'), ('LA', "Lao People's Democratic Republic"), ('LV', 'Latvia'), ('LB', 'Lebanon'), ('LS', 'Lesotho'), ('LR', 'Liberia'), ('LY', 'Libyan Arab Jamahiriya'), ('LI', 'Liechtenstein'), ('LT', 'Lithuania'), ('LU', 'Luxembourg'), ('MO', 'Macao'), ('MK', 'Macedonia'), ('MG', 'Madagascar'), ('MW', 'Malawi'), ('MY', 'Malaysia'), ('MV', 'Maldives'), ('ML', 'Mali'), ('ML', 'Malta'), ('MH', 'Marshall Islands'), ('MQ', 'Martinique'), ('MR', 'Mauritania'), ('MU', 'Mauritius'), ('YT', 'Mayotte'), ('MX', 'Mexico'), ('FM', 'Micronesia'), ('MD', 'Moldova'), ('MC', 'Monaco'), ('MN', 'Mongolia'), ('ME', 'Montenegro'), ('MS', 'Montserrat'), ('MA', 'Morocco'), ('MZ', 'Mozambique'), ('MM', 'Myanmar'), ('NA', 'Namibia'), ('NR', 'Nauru'), ('NP', 'Nepal'), ('NL', 'Netherlands'), ('AN', 'Netherlands Antilles'), ('NC', 'New Caledonia'), ('NZ', 'New Zealand'), ('NI', 'Nicaragua'), ('NE', 'Niger'), ('NG', 'Nigeria'), ('NU', 'Niue'), ('NF', 'Norfolk Island'), ('MP', 'Northern Mariana Islands'), ('NO', 'Norway'), ('OM', 'Oman'), ('PK', 'Pakistan'), ('PW', 'Palau'), ('PS', 'Palestinian Territory, Occupied'), ('PA', 'Panama'), ('PG', 'Papua New Guinea'), ('PY', 'Paraguay'), ('PE', 'Peru'), ('PH', 'Philippines'), ('PN', 'Pitcairn'), ('PL', 'Poland'), ('PT', 'Portugal'), ('PR', 'Puerto Rico'), ('QA', 'Qatar'), ('RE', 'Reunion'), ('RO', 'Romania'), ('RU', 'Russian Federation'), ('RW', 'Rwanda'), ('BL', 'Saint Barthelemy'), ('SH', 'Saint Helena, Ascension & Tristan Da Cunha'), ('KN', 'Saint Kitts and Nevis'), ('LC', 'Saint Lucia'), ('MF', 'Saint Martin (French Part)'), ('PM', 'Saint Pierre and Miquelon'), ('VC', 'Saint Vincent And The Grenadines'), ('WS', 'Samoa'), ('SM', 'San Marino'), ('ST', 'Sao Tome And Principe'), ('SA', 'Saudi Arabia'), ('SN', 'Senegal'), ('RS', 'Serbia'), ('SC', 'Seychelles'), ('SL', 'Sierra Leone'), ('SG', 'Singapore'), ('SX', 'Sint Maarten (Dutch Part)'), ('SK', 'Slovakia'), ('SI', 'Slovenia'), ('SB', 'Solomon Islands'), ('SO', 'Somalia'), ('ZA', 'South Africa'), ('GS', 'South Georgia And The South Sandwich Islands'), ('ES', 'Spain'), ('LK', 'Sri Lanka'), ('SD', 'Sudan'), ('SR', 'Suriname'), ('SJ', 'Svalbard And Jan Mayen'), ('SZ', 'Swaziland'), ('SE', 'Sweden'), ('CH', 'Switzerland'), ('SY', 'Syrian Arab Republic'), ('TW', 'Taiwan'), ('TJ', 'Tajikistan'), ('TZ', 'Tanzania'), ('TH', 'Thailand'), ('TL', 'Timor-Leste'), ('TG', 'Togo'), ('TK', 'Tokelau'), ('TO', 'Tonga'), ('TT', 'Trinidad and Tobago'), ('TN', 'Tunisia'), ('TR', 'Turkey'), ('TM', 'Turkmenistan'), ('TC', 'Turks And Caicos Islands'), ('TV', 'Tuvalu'), ('UG', 'Uganda'), ('UA', 'Ukraine'), ('AE', 'United Arab Emirates'), ('GB', 'United Kingdom'), ('US', 'United States'), ('UM', 'United States Minor Outlying Islands'), ('UY', 'Uruguay'), ('UZ', 'Uzbekistan'), ('VU', 'Vanuatu'), ('VE', 'Venezuela, Bolivarian Republic Of'), ('VN', 'Viet Nam'), ('VG', 'Virgin Islands, British'), ('VI', 'Virgin Islands, U.S.'), ('WF', 'Wallis and Futuna'), ('EH', 'Western Sahara'), ('YE', 'Yemen'), ('ZM', 'Zambia'), ('ZW', 'Zimbabwe')], max_length=3, verbose_name='Country')),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopit.Customer')),
            ],
            options={
                'db_table': 'shopit_shipping_addresses',
                'verbose_name': 'Shipping address',
                'verbose_name_plural': 'Shipping addresses',
            },
        ),
        migrations.CreateModel(
            name='Tax',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('percent', models.DecimalField(decimal_places=2, help_text='Tax percentage.', max_digits=4, validators=[django.core.validators.MinValueValidator(Decimal('0.00'))], verbose_name='Percent')),
                ('order', models.PositiveIntegerField(default=0, verbose_name='Sort')),
            ],
            options={
                'ordering': ['order'],
                'db_table': 'shopit_taxes',
                'verbose_name': 'Tax',
                'verbose_name_plural': 'Taxes',
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='TaxTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language_code', models.CharField(db_index=True, max_length=15, verbose_name='Language')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('master', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='shopit.Tax')),
            ],
            options={
                'managed': True,
                'db_table': 'shopit_taxes_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'Tax Translation',
            },
        ),
        migrations.AddField(
            model_name='product',
            name='_tax',
            field=models.ForeignKey(blank=True, help_text='Tax to be applied to this product. Variants inherit tax percentage from their Group, and should leave this field empty.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='shopit.Tax', verbose_name='Tax'),
        ),
        migrations.AddField(
            model_name='product',
            name='attributes',
            field=models.ManyToManyField(through='shopit.AttributeValue', to='shopit.Attribute', verbose_name='Attributes'),
        ),
        migrations.AddField(
            model_name='product',
            name='available_attributes',
            field=models.ManyToManyField(blank=True, help_text='Select attributes that can be used in a Variant for this product.', related_name='products_available_attributes', to='shopit.Attribute', verbose_name='Attributes'),
        ),
        migrations.AddField(
            model_name='product',
            name='content',
            field=cms.models.fields.PlaceholderField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, slotname='shopit_product_content', to='cms.Placeholder'),
        ),
        migrations.AddField(
            model_name='product',
            name='flags',
            field=models.ManyToManyField(blank=True, help_text='Check flags for this product.', to='shopit.Flag', verbose_name='Flags'),
        ),
        migrations.AddField(
            model_name='product',
            name='group',
            field=models.ForeignKey(blank=True, help_text='Select a Group product for this variation.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='variants', to='shopit.Product', verbose_name='Group'),
        ),
        migrations.AddField(
            model_name='product',
            name='modifiers',
            field=models.ManyToManyField(blank=True, to='shopit.Modifier', verbose_name='Modifiers'),
        ),
        migrations.AddField(
            model_name='product',
            name='polymorphic_ctype',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_shopit.product_set+', to='contenttypes.ContentType'),
        ),
        migrations.AddField(
            model_name='orderitem',
            name='product',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='shopit.Product', verbose_name='Product'),
        ),
        migrations.AddField(
            model_name='manufacturer',
            name='modifiers',
            field=models.ManyToManyField(blank=True, to='shopit.Modifier', verbose_name='Modifiers'),
        ),
        migrations.AddField(
            model_name='manufacturer',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='shopit.Manufacturer', verbose_name='Parent'),
        ),
        migrations.AddField(
            model_name='discountcode',
            name='modifier',
            field=models.ForeignKey(help_text='Modifier that this discount code applies to.', on_delete=django.db.models.deletion.CASCADE, related_name='discount_codes', to='shopit.Modifier', verbose_name='Modifier'),
        ),
        migrations.AddField(
            model_name='deliveryitem',
            name='item',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopit.OrderItem', verbose_name='Ordered item'),
        ),
        migrations.AddField(
            model_name='delivery',
            name='order',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopit.Order'),
        ),
        migrations.AddField(
            model_name='category',
            name='_tax',
            field=models.ForeignKey(blank=True, help_text="Tax to be applied to products in this category. If empty, parent's tax will be used.", null=True, on_delete=django.db.models.deletion.SET_NULL, to='shopit.Tax', verbose_name='Tax'),
        ),
        migrations.AddField(
            model_name='category',
            name='flags',
            field=models.ManyToManyField(blank=True, help_text='Check flags for products in this categorization.', to='shopit.Flag', verbose_name='Flags'),
        ),
        migrations.AddField(
            model_name='category',
            name='modifiers',
            field=models.ManyToManyField(blank=True, to='shopit.Modifier', verbose_name='Modifiers'),
        ),
        migrations.AddField(
            model_name='category',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='shopit.Category', verbose_name='Parent'),
        ),
        migrations.AddField(
            model_name='cartitem',
            name='product',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopit.Product'),
        ),
        migrations.AddField(
            model_name='cart',
            name='customer',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='cart', to='shopit.Customer', verbose_name='Customer'),
        ),
        migrations.AddField(
            model_name='cart',
            name='shipping_address',
            field=models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='shopit.ShippingAddress'),
        ),
        migrations.AddField(
            model_name='brand',
            name='flags',
            field=models.ManyToManyField(blank=True, help_text='Check flags for products in this categorization.', to='shopit.Flag', verbose_name='Flags'),
        ),
        migrations.AddField(
            model_name='brand',
            name='modifiers',
            field=models.ManyToManyField(blank=True, to='shopit.Modifier', verbose_name='Modifiers'),
        ),
        migrations.AddField(
            model_name='brand',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='shopit.Brand', verbose_name='Parent'),
        ),
        migrations.AddField(
            model_name='billingaddress',
            name='customer',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopit.Customer'),
        ),
        migrations.AddField(
            model_name='attributevalue',
            name='product',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attribute_values', to='shopit.Product', verbose_name='Product'),
        ),
        migrations.AddField(
            model_name='attachment',
            name='product',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='shopit.Product', verbose_name='Product'),
        ),
        migrations.AlterUniqueTogether(
            name='taxtranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='reviewtranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='relation',
            unique_together=set([('base', 'product', 'kind')]),
        ),
        migrations.AlterUniqueTogether(
            name='producttranslation',
            unique_together=set([('language_code', 'master'), ('language_code', 'slug')]),
        ),
        migrations.AlterUniqueTogether(
            name='modifiertranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='manufacturertranslation',
            unique_together=set([('language_code', 'master'), ('language_code', 'slug')]),
        ),
        migrations.AlterUniqueTogether(
            name='flagtranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='categorytranslation',
            unique_together=set([('language_code', 'master'), ('language_code', 'slug')]),
        ),
        migrations.AlterUniqueTogether(
            name='brandtranslation',
            unique_together=set([('language_code', 'master'), ('language_code', 'slug')]),
        ),
        migrations.AlterUniqueTogether(
            name='attributevalue',
            unique_together=set([('attribute', 'product')]),
        ),
        migrations.AlterUniqueTogether(
            name='attributetranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='attributechoicetranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='attributechoice',
            unique_together=set([('attribute', 'value')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.db import models
from django.db.models import Q
from django.utils.encoding import python_2_unicode_compatible, smart_text
from django.utils.translation import ugettext_lazy as _
from mptt.managers import TreeManager
from mptt.models import MPTTModel, TreeForeignKey
from mptt.querysets import TreeQuerySet
from parler.managers import TranslatableManager, TranslatableQuerySet
from parler.models import TranslatableModelMixin, TranslatedFields

from shopit.conf import app_settings


class FlagQuerySet(TranslatableQuerySet, TreeQuerySet):
    def active(self):
        return self.filter(Q(active=True) & (Q(parent__isnull=True) | Q(parent__active=True)))


class FlagManager(TranslatableManager, TreeManager):
    queryset_class = FlagQuerySet

    def get_queryset(self):
        return self.queryset_class(self.model, using=self._db).order_by('tree_id', 'lft')

    def active(self):
        return self.get_queryset().active()


@python_2_unicode_compatible
class Flag(TranslatableModelMixin, MPTTModel):
    """
    Flag model.
    """
    TEMPLATES = app_settings.FLAG_TEMPLATES

    translations = TranslatedFields(
        name=models.CharField(
            _('Name'),
            max_length=128,
        ),
    )

    code = models.SlugField(
        _('Code'),
        unique=True,
        help_text=_('Unique identifier for this flag.'),
    )

    parent = TreeForeignKey(
        'self',
        models.CASCADE,
        blank=True,
        null=True,
        related_name='children',
        verbose_name=_('Parent'),
    )

    template = models.CharField(
        _('Template'),
        max_length=255,
        blank=True,
        null=True,
        choices=TEMPLATES,
        help_text=_(
            'You can specify a template for rendering this flag or leave it empty for the '
            'default look.'
        ),
    )

    active = models.BooleanField(
        _('Active'),
        default=True,
        help_text=_('Is this flag publicly visible.'),
    )

    created_at = models.DateTimeField(
        _('Created at'),
        auto_now_add=True,
    )

    updated_at = models.DateTimeField(
        _('Updated at'),
        auto_now=True,
    )

    objects = FlagManager()

    class Meta:
        db_table = 'shopit_flags'
        verbose_name = _('Flag')
        verbose_name_plural = _('Flags')
        ordering = ['tree_id', 'lft']

    def __str__(self):
        name = self.safe_translation_getter('name', any_language=True)
        return '%s | %s' % (smart_text(self.parent), name) if self.parent else name

    def get_products(self):
        """
        Returs all flagged products with this flag.
        """
        products = getattr(self, '_products', None)
        if products is None:
            products = self.product_set.active()
            setattr(self, '_products', products)
        return products



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _
from shop.money import Money

from shopit.conf import app_settings


class ModifierCondition(object):
    """
    A base model for creating custom modifier conditions.
    """
    def __init__(self, name=None):
        self.name = name or getattr(self, 'name', self.__class__.__name__)

    def cart_item_condition(self, request, cart_item, value=None):
        """
        Returns if condition is met on a cart item.
        """
        return True

    def cart_condition(self, request, cart, value=None):
        """
        Returns if condition is met on a cart.
        """
        return True


class PriceGreaterThanCondition(ModifierCondition):
    name = _('Price greater than')

    def cart_item_condition(self, request, cart_item, value=0):
        return cart_item.line_total > Money(value)

    def cart_condition(self, request, cart, value=0):
        return cart.total > Money(value)


class PriceLessThanCondition(ModifierCondition):
    name = _('Price less than')

    def cart_item_condition(self, request, cart_item, value=0):
        return cart_item.line_total < Money(value)

    def cart_condition(self, request, cart, value=0):
        return cart.total < Money(value)


class QuantityGreaterThanCondition(ModifierCondition):
    name = _('Quantity greater than')

    def cart_item_condition(self, request, cart_item, value=0):
        return cart_item.quantity > int(value)


class QuantityLessThanCondition(ModifierCondition):
    name = _('Quantity less than')

    def cart_item_condition(self, request, cart_item, value=0):
        return cart_item.quantity < int(value)


class ModifierConditionsPool(object):
    """
    A pool that keeps all available modifier conditions.
    """
    def get_all_conditions(self):
        if not hasattr(self, '_all_conditions'):
            setattr(self, '_all_conditions', [import_string(path)() for path in app_settings.MODIFIER_CONDITIONS])
        return getattr(self, '_all_conditions')

    def get_condition_choices(self):
        if not hasattr(self, '_condition_choices'):
            choices = [('%s.%s' % (x.__module__, x.__class__.__name__), x.name) for x in self.get_all_conditions()]
            setattr(self, '_condition_choices', choices)
        return getattr(self, '_condition_choices')

    def get_condition(self, path):
        if path in app_settings.MODIFIER_CONDITIONS:
            for condition in self.get_all_conditions():
                if path == '%s.%s' % (condition.__module__, condition.__class__.__name__):
                    return condition


modifier_conditions_pool = ModifierConditionsPool()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from adminsortable2.admin import CustomInlineFormSet as SortableInlineFormSet
from django import forms
from django.forms.models import BaseInlineFormSet
from django.utils.module_loading import import_string
from parler.forms import TranslatableModelForm

from shopit.conf import app_settings
from shopit.models.product import AttributeValue, Product
from shopit.utils import get_error_message as em

try:
    TextEditor = import_string(app_settings.TEXT_EDITOR)
except ImportError:  # pragma: no cover
    from django.forms.widgets import Textarea as TextEditor


class ProductModelForm(TranslatableModelForm):
    class Meta:
        model = Product
        exclude = []
        widgets = {
            '_caption': forms.Textarea(attrs={'rows': 2}),
            '_description': TextEditor(),
        }

    def clean_slug(self):
        slug = self.cleaned_data.get('slug')
        queryset = Product.objects.translated(slug=slug)
        if self.instance.pk is not None:
            queryset = queryset.exclude(pk=self.instance.pk)
        if queryset.exists():
            raise forms.ValidationError(em('duplicate_slug'))
        return slug

    def clean_kind(self):
        kind = self.cleaned_data.get('kind')
        if kind != Product.GROUP:
            if self.instance.variants.exists():
                raise forms.ValidationError(em('not_group_has_variants'))
        return kind

    def clean_category(self):
        return self._clean_categorization('category')  # pragma: no cover

    def clean_brand(self):
        return self._clean_categorization('brand')  # pragma: no cover

    def clean_manufacturer(self):
        return self._clean_categorization('manufacturer')  # pragma: no cover

    def clean__tax(self):
        tax = self.cleaned_data.get('_tax')
        kind = self.cleaned_data.get('kind')
        if kind == Product.VARIANT and tax is not None:
            raise forms.ValidationError(em('variant_has_tax'))
        return tax

    def clean_group(self):
        group = self.cleaned_data.get('group')
        kind = self.cleaned_data.get('kind')
        if group is None:
            if kind == Product.VARIANT:
                raise forms.ValidationError(em('variant_no_group'))
        else:
            if kind != Product.VARIANT:
                raise forms.ValidationError(em('group_has_group'))
            if group.is_variant:
                raise forms.ValidationError(em('varinat_group_variant'))
        return group

    def clean_available_attributes(self):
        attrs = self.cleaned_data.get('available_attributes')
        kind = self.cleaned_data.get('kind')
        if kind != Product.GROUP and attrs:
            raise forms.ValidationError(em('not_group_has_available_attributes'))
        elif kind == Product.GROUP and not attrs:
            raise forms.ValidationError(em('group_no_available_attributes'))
        return attrs

    def _clean_categorization(self, name):
        data = self.cleaned_data.get(name, None)
        kind = self.cleaned_data.get('kind')
        if data and kind == Product.VARIANT:
            raise forms.ValidationError(em('variant_has_category'))
        return data


class AttributeChoiceInlineFormSet(SortableInlineFormSet):
    def clean(self):
        super(AttributeChoiceInlineFormSet, self).clean()
        instance = getattr(self, 'instance', None)

        if any(self.errors) or instance is None:
            return

        clean_forms = [x for x in self.forms if 'value' in x.cleaned_data and not x.cleaned_data['DELETE']]
        if not clean_forms:
            raise forms.ValidationError(em('attribute_no_choices'))

        clean_values = [x.cleaned_data['value'] for x in clean_forms]
        if len(clean_values) != len(set(clean_values)):
            raise forms.ValidationError(em('attribute_duplicate_choices'))


class AttributeValueInlineFormSet(BaseInlineFormSet):
    def clean(self):
        super(AttributeValueInlineFormSet, self).clean()
        instance = getattr(self, 'instance', None)

        if any(self.errors) or instance is None:
            return

        clean_forms = [x for x in self.forms if 'attribute' in x.cleaned_data and not x.cleaned_data['DELETE']]

        if instance.is_variant:
            if not clean_forms:
                raise forms.ValidationError(em('variant_no_attributes'))
            elif self.variant_exists(instance, clean_forms):
                raise forms.ValidationError(em('variant_already_exists'))
        elif clean_forms:
            raise forms.ValidationError(em('not_variant_has_attributes'))

    def variant_exists(self, variant, forms):
        """
        Checks if variant with this attributes already exists.
        """
        variations = [dict(x[1]) for x in variant.group.get_variations() if x[0] != variant.pk]

        if variations:
            attrs = {}
            for form in forms:
                del form.cleaned_data['DELETE']
                key = form.cleaned_data['attribute'].key
                attrs[key] = AttributeValue(**form.cleaned_data).as_dict
            return attrs in variations


class AttributeValueModelForm(forms.ModelForm):
    class Meta:
        model = AttributeValue
        fields = ['attribute', 'choice']

    def clean_choice(self):
        choice = self.cleaned_data.get('choice')
        attribute = self.cleaned_data.get('attribute')

        # Make sure correct choice is selected for the attribute.
        if attribute and not attribute.nullable:
            if choice not in attribute.get_choices():
                raise forms.ValidationError(em('incorrect_attribute_choice'))
        return choice



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-04-04 10:19
from __future__ import unicode_literals

from decimal import Decimal
from django.db import migrations, models
import shop.money.fields


class Migration(migrations.Migration):

    dependencies = [
        ('shopit', '0010_add_placeholder_fields_to_categorization_models'),
    ]

    operations = [
        migrations.AlterField(
            model_name='modifier',
            name='amount',
            field=shop.money.fields.MoneyField(default=0, help_text='Amount that should be added. Can be negative.', verbose_name='Amount'),
        ),
        migrations.AlterField(
            model_name='product',
            name='_unit_price',
            field=shop.money.fields.MoneyField(default=0, help_text='For variants leave empty to use the Group price.', verbose_name='Unit price'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-12-19 13:16
from __future__ import unicode_literals

from django.db import migrations
import mptt
import mptt.managers


def rebuild_flag_mptt_tree(apps, schema_editor):
    manager = mptt.managers.TreeManager()
    Flag = apps.get_model('shopit', 'Flag')
    manager.model = Flag
    mptt.register(Flag, order_insertion_by=['created_at'])
    manager.contribute_to_class(Flag, 'objects')
    manager.rebuild()


class Migration(migrations.Migration):

    dependencies = [
        ('shopit', '0002_make_flag_mptt_model'),
    ]

    operations = [
        migrations.RunPython(rebuild_flag_mptt_tree, lambda apps, schema_editor: None)
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from shop.models.customer import BaseCustomer


@python_2_unicode_compatible
class Customer(BaseCustomer):
    SALUTATION = [
        ('mrs', _("Mrs.")),
        ('mr', _("Mr.")),
        ('na', _("(n/a)")),
    ]

    number = models.PositiveIntegerField(
        _('Customer Number'),
        null=True,
        default=None,
        unique=True,
    )

    salutation = models.CharField(
        _('Salutation'),
        max_length=5,
        choices=SALUTATION,
    )

    phone_number = models.CharField(
        _('Phone number'),
        max_length=255,
        blank=True,
    )

    class Meta:
        db_table = 'shopit_customers'
        verbose_name = _('Customer')
        verbose_name_plural = _('Customers')

    def __str__(self):
        return self.get_username()

    def get_number(self):
        return self.number

    def get_or_assign_number(self):
        if self.number is None:
            aggr = Customer.objects.filter(number__isnull=False).aggregate(models.Max('number'))
            self.number = (aggr['number__max'] or 0) + 1
            self.save()
        return self.get_number()

    def get_discount_codes(self):
        if not hasattr(self, '_discount_codes'):
            setattr(self, '_discount_codes', self.discount_codes.valid())
        return getattr(self, '_discount_codes')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django import forms
from django.contrib.auth import authenticate, get_user_model, login, update_session_auth_hash
from django.contrib.auth.forms import AuthenticationForm, PasswordChangeForm, UserCreationForm
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from shopit.conf import app_settings
from shopit.models.customer import Customer


class CleanEmailMixin(object):
    """
    Mixin used in forms where customers email is modified or created.
    Checks that email doesn't already exists.
    """
    def clean_email(self):
        email = self.cleaned_data['email']
        if 'email' in self.changed_data and get_user_model().objects.filter(is_active=True, email=email).exists():
            raise ValidationError(_('An account with this email address already exists.'))
        return email


class AccountLoginForm(AuthenticationForm):
    """
    Override username to be an email field, to disallow username logins.
    """
    username = forms.EmailField(label=_('Email address'))


class AccountRegisterForm(CleanEmailMixin, UserCreationForm):
    email = forms.EmailField(label=_('Email address'))

    class Meta:
        model = Customer
        fields = ['email', 'password1', 'password2']

    def __init__(self, *args, **kwargs):
        # Override default init from `UserCreationForm` to use email as
        # username field, and set autofocus.
        super(UserCreationForm, self).__init__(*args, **kwargs)
        self.fields['email'].widget.attrs.update({'autofocus': ''})

    def save(self, request, commit=False):
        self.instance = Customer.objects.get_from_request(request)
        self.instance.recognize_as_registered()
        self.instance.user.is_active = True
        self.instance.user.email = self.cleaned_data['email']
        password = self.cleaned_data['password1']
        self.instance.user.set_password(password)
        self.instance.save()
        user = authenticate(username=self.instance.user.username, password=password)
        login(request, user)
        return self.instance


class AccountDetailsForm(CleanEmailMixin, forms.ModelForm):
    first_name = forms.CharField(
        label=_('First name'),
        max_length=30,
        required=False,
    )

    last_name = forms.CharField(
        label=_('Last name'),
        max_length=30,
        required=False,
    )

    email = forms.EmailField(label=_('Email address'))

    class Meta:
        model = Customer
        fields = ['salutation', 'first_name', 'last_name', 'email', 'phone_number']
        custom_fields = ['first_name', 'last_name', 'email']

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request')
        self.instance = Customer.objects.get_from_request(self.request)
        kwargs['instance'] = self.instance
        kwargs['initial'] = dict([(x, getattr(self.instance, x)) for x in self.Meta.custom_fields])
        super(AccountDetailsForm, self).__init__(*args, **kwargs)
        self.fields['phone_number'].required = app_settings.PHONE_NUMBER_REQUIRED

    def save(self, commit=True):
        for field in self.Meta.custom_fields:
            setattr(self.instance, field, self.cleaned_data[field])
        customer = super(AccountDetailsForm, self).save(commit)
        return customer

    def get_success_message(self):
        return _('Info successfully updated.')  # pragma: no cover


class AccountPasswordForm(PasswordChangeForm):
    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request')
        super(AccountPasswordForm, self).__init__(self.request.user, *args, **kwargs)

    def save(self, commit=True):
        user = super(AccountPasswordForm, self).save(commit)
        update_session_auth_hash(self.request, user)
        return user

    def get_success_message(self):
        return _('Password successfully changed.')  # pragma: no cover



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

"""
Release logic:
 1. Bump the __version__.
 2. git add shopit/__init__.py
 3. git commit -m 'Bump to <version>'
 4. git push
 5. Make sure all tests pass on https://travis-ci.com/dinoperovic/django-shopit
 6. git tag <version>
 7. git push --tags
 8. python setup.py sdist bdist_wheel
 9. twine upload dist/*
10. Done!
"""
__version__ = '0.5.1'

default_app_config = 'shopit.apps.ShopitConfig'



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import itertools
from collections import OrderedDict
from datetime import datetime
from decimal import Decimal
from os.path import basename

from cms.models.fields import PlaceholderField
from cms.utils.i18n import get_current_language
from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.urlresolvers import NoReverseMatch, reverse
from django.core.validators import MinValueValidator
from django.db import models, transaction
from django.db.models import Count
from django.db.models.query import QuerySet
from django.template.defaultfilters import truncatewords
from django.utils import timezone
from django.utils.decorators import method_decorator
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _
from django_measurement.models import MeasurementField
from easy_thumbnails.exceptions import InvalidImageFormatError
from filer.fields.file import FilerFileField
from measurement.measures import Distance, Mass
from mptt.fields import TreeForeignKey
from parler.managers import TranslatableManager, TranslatableQuerySet
from parler.models import TranslatableModel, TranslatedFields
from parler.utils.context import switch_language
from polymorphic.query import PolymorphicQuerySet
from shop.models.product import BaseProduct, BaseProductManager
from shop.money.fields import MoneyField

from shopit.conf import app_settings
from shopit.models.cart import Cart
from shopit.models.categorization import Brand, Category, Manufacturer
from shopit.models.customer import Customer
from shopit.models.flag import Flag
from shopit.models.modifier import Modifier
from shopit.models.tax import Tax
from shopit.utils import get_error_message as em

try:
    from easy_thumbnails.files import get_thumbnailer
    from easy_thumbnails.alias import aliases
    EASY_THUMBNAILS = True
except ImportError:  # pragma: no cover
    EASY_THUMBNAILS = False


class ProductQuerySet(PolymorphicQuerySet, TranslatableQuerySet):
    def active(self):
        return self.filter(active=True)

    def top_level(self):
        return self.filter(kind__in=[Product.SINGLE, Product.GROUP])

    def filter_categorization(self, categories=None, brands=None, manufacturers=None):
        """
        Filters a queryset by the given categorization. A list of slugs should
        be passed in.
        """
        filters = {}
        if categories:
            ids = Category.objects.translated(slug__in=categories).values_list('id', flat=True)
            filters['_category_id__in'] = list(set(ids))
        if brands:
            ids = Brand.objects.translated(slug__in=brands).values_list('id', flat=True)
            filters['_brand_id__in'] = list(set(ids))
        if manufacturers:
            ids = Manufacturer.objects.translated(slug__in=manufacturers).values_list('id', flat=True)
            filters['_manufacturer_id__in'] = list(set(ids))
        return self.filter(**filters) if filters else self

    def filter_flags(self, flags=None):
        """
        Filters a queryset by the given flags. A list of codes should be
        passed in.
        """
        filters = {}
        if flags:
            flagged = self.prefetch_related('flags').filter(flags__code__in=flags)
            flagged = flagged.annotate(num_flags=Count('flags')).filter(num_flags=len(flags)).distinct()
            filters['id__in'] = flagged.values_list('id', flat=True)
        return self.filter(**filters) if filters else self

    def filter_modifiers(self, modifiers=None):
        """
        Filters a queryset by the given modifiers. A list of codes should be
        passed in.
        """
        filters = {}
        if modifiers:
            enabled = Modifier.objects.filtering_enabled().active().values_list('code', flat=True)
            if len([x for x in modifiers if x in enabled]) < len(modifiers):
                # Return empty queryset if invalid modifiers are passed in.
                return self.none()

            modded = self.prefetch_related('modifiers').filter(modifiers__code__in=modifiers)
            modded = modded.annotate(num_mods=Count('modifiers')).filter(num_mods=len(modifiers)).distinct()
            filters['id__in'] = modded.values_list('id', flat=True)
        return self.filter(**filters) if filters else self

    def filter_attributes(self, attributes=None):
        """
        Filters a queryset by attributes. A list of tuples containing attribute
        code, name should be passed in as `attributes`.
        """
        if attributes:
            ids = self.values_list('id', flat=True)
            variants = Product.objects.filter(group_id__in=ids).\
                prefetch_related('attribute_values', 'attribute_values__attribute', 'attribute_values__choice')

            if variants:
                for code, value in attributes:
                    filters = {'attribute_values__attribute__code__iexact': code}
                    if value:
                        filters['attribute_values__choice__value__iexact'] = value
                    else:
                        filters['attribute_values__choice__isnull'] = True
                    variants = variants.filter(**filters)

                group_ids = list(set(variants.values_list('group_id', flat=True)))
                self = self.filter(id__in=group_ids)

                if app_settings.FILTER_ATTRIBUTES_INCLUDES_VARIANTS:
                    self = (variants | self).order_by('-order', 'kind', 'published')
        return self

    def filter_price(self, price_from=None, price_to=None):
        """
        Filters a queryset by a price range.
        """
        filters = {}
        if price_from:
            filters['_unit_price__gte'] = Decimal(price_from)
        if price_to:
            filters['_unit_price__lte'] = Decimal(price_to)
        return self.filter(**filters) if filters else self


class ProductManager(BaseProductManager, TranslatableManager):
    queryset_class = ProductQuerySet

    def active(self):
        return self.get_queryset().active()

    def top_level(self):
        return self.get_queryset().top_level()

    def filter_categorization(self, categories=None, brands=None, manufacturers=None):
        return self.get_queryset().filter_categorization(categories, brands, manufacturers)

    def filter_flags(self, flags=None):
        return self.get_queryset().filter_flags(flags)

    def filter_modifiers(self, modifiers=None):
        return self.get_queryset().filter_modifiers(modifiers)

    def filter_price(self, price_from=None, price_to=None):
        return self.get_queryset().filter_price(price_from, price_to)

    def filter_attributes(self, attributes=None):
        return self.get_queryset().filter_attributes(attributes)


@python_2_unicode_compatible
class Product(BaseProduct, TranslatableModel):
    """
    Product model.
    """
    SINGLE, GROUP, VARIANT = range(3)

    KINDS = (
        (SINGLE, _('Single')),
        (GROUP, _('Group')),
        (VARIANT, _('Variant')),
    )

    translations = TranslatedFields(
        name=models.CharField(
            _('Name'),
            max_length=128,
        ),
        slug=models.SlugField(
            _('Slug'),
            db_index=True,
            help_text=_("Part that's used in url to display this product. Needs to be unique."),
        ),
        _caption=models.TextField(
            _('Caption'),
            max_length=255,
            blank=True,
            help_text=_("Short product caption, usually used in catalog's list view of products."),
        ),
        _description=models.TextField(
            _('Description'),
            blank=True,
            help_text=_("Description of a product, usually used as lead text in product's detail view."),
        ),
        meta={
            'unique_together': [('language_code', 'slug')],
        },
    )

    code = models.CharField(
        _('Code'),
        max_length=64,
        unique=True,
        help_text=_('Unique identifier for a product.'),
    )

    # Categorization
    _category = TreeForeignKey(
        Category,
        models.CASCADE,
        blank=True,
        null=True,
        verbose_name=_('Category'),
    )

    _brand = TreeForeignKey(
        Brand,
        models.CASCADE,
        blank=True,
        null=True,
        verbose_name=_('Brand'),
    )

    _manufacturer = TreeForeignKey(
        Manufacturer,
        models.CASCADE,
        blank=True,
        null=True,
        verbose_name=_('Manufacturer'),
    )

    # Pricing
    _unit_price = MoneyField(
        _('Unit price'),
        default=0,
        help_text=_("For variants leave empty to use the Group price."),
    )

    _discount = models.DecimalField(
        _('Discount %'),
        blank=True,
        null=True,
        max_digits=4,
        decimal_places=2,
        validators=[MinValueValidator(Decimal('0.00'))],
        help_text=_("For variants leave empty to use Group discount."),
    )

    _tax = models.ForeignKey(
        Tax,
        models.SET_NULL,
        blank=True,
        null=True,
        verbose_name=_('Tax'),
        help_text=_(
            "Tax to be applied to this product. Variants inherit tax percentage from their Group, and should "
            "leave this field empty."
        ),
    )

    # Settings
    kind = models.PositiveSmallIntegerField(
        _('Kind'),
        choices=KINDS,
        default=SINGLE,
        help_text=_(
            'Choose a product type. Single products are products without variations. Group products are base products '
            'that hold variants and their common info, they cannot be added to cart. Variants are variations of a '
            'product that must select a Group product, and set their unique set of attributes. '
            '(See "Variant" section below)'
        ),
    )

    discountable = models.BooleanField(
        _('Discountable'),
        default=True,
        help_text=_('Can this product be used in an offer?'),
    )

    modifiers = models.ManyToManyField(
        Modifier,
        blank=True,
        verbose_name=_('Modifiers'),
        limit_choices_to={'kind__in': [Modifier.STANDARD, Modifier.DISCOUNT]},
    )

    flags = models.ManyToManyField(
        Flag,
        blank=True,
        verbose_name=_('Flags'),
        help_text=_('Check flags for this product.'),
    )

    # Measurements
    _width = MeasurementField(
        _('Width'),
        blank=True,
        null=True,
        measurement=Distance,
    )

    _height = MeasurementField(
        _('Height'),
        blank=True,
        null=True,
        measurement=Distance,
    )

    _depth = MeasurementField(
        _('Depth'),
        blank=True,
        null=True,
        measurement=Distance,
    )

    _weight = MeasurementField(
        _('Weight'),
        blank=True,
        null=True,
        measurement=Mass,
    )

    # Group
    available_attributes = models.ManyToManyField(
        'Attribute',
        blank=True,
        related_name='products_available_attributes',
        verbose_name=_('Attributes'),
        help_text=_('Select attributes that can be used in a Variant for this product.'),
    )

    # Variant
    group = models.ForeignKey(
        'self',
        models.CASCADE,
        blank=True,
        null=True,
        related_name='variants',
        verbose_name=_('Group'),
        help_text=_('Select a Group product for this variation.'),
    )

    attributes = models.ManyToManyField(
        'Attribute',
        through='AttributeValue',
        verbose_name=_('Attributes'),
    )

    published = models.DateTimeField(
        _('Published'),
        default=timezone.now,
    )

    quantity = models.IntegerField(
        _('Quantity'),
        blank=True,
        null=True,
        help_text=_(
            'Number of available products to ship. Leave empty if product is always available, or set to 0 if product '
            'is not available.'
        ),
    )

    order = models.BigIntegerField(
        _('Sort'),
        default=0,
    )

    content = PlaceholderField('shopit_product_content')

    objects = ProductManager()

    lookup_fields = ['code__startswith', 'translations__name__icontains']

    class Meta:
        db_table = 'shopit_products'
        verbose_name = _('Product')
        verbose_name_plural = _('Products')
        ordering = ['-order']

    def __str__(self):
        return self.product_name

    def save(self, *args, **kwargs):
        """
        Clean and clear product.
        Set unique ordering value for product and it's variants based on
        published timestamp. Force Single groups to a Group kind.
        """
        self.clean()
        self.clear()
        if self.is_variant:
            self.group.clear('_variants', '_invalid_variants', '_variations', '_attribute_choices', '_combinations')
            self.order = self.group.order
            if self.group.is_single:
                self.group.kind = Product.GROUP
                self.group.save(update_fields=['kind'])
            super(Product, self).save(*args, **kwargs)
        else:
            # Don't generate timestamp if published hasn't changed.
            if self.pk is not None:
                original = Product.objects.get(pk=self.pk).published.strftime('%s')
                if original == self.published.strftime('%s'):
                    super(Product, self).save(*args, **kwargs)
                    return
            timestamp = int(self.published.strftime('%s'))
            same = Product.objects.filter(order__startswith=timestamp).first()
            timestamp = same.order + 1 if same else timestamp * 1000
            self.order = timestamp
            super(Product, self).save(*args, **kwargs)
            if self.is_group:
                for variant in self.get_variants():
                    variant.clear()
                    variant.order = timestamp
                    variant.save(update_fields=['order'])

    def get_absolute_url(self, language=None):
        if not language:
            language = get_current_language()

        with switch_language(self, language):
            try:
                return reverse('shopit-product-detail', args=[self.safe_translation_getter('slug')])
            except NoReverseMatch:  # pragma: no cover
                pass

    @property
    def product_name(self):
        return self.safe_translation_getter('name', any_language=True) if self.pk else ''

    @property
    def product_code(self):
        return self.code

    @property
    def is_single(self):
        return self.kind == self.SINGLE

    @property
    def is_group(self):
        return self.kind == self.GROUP

    @property
    def is_variant(self):
        return self.kind == self.VARIANT

    @property
    def caption(self):
        return self.get_attr('_caption', '', translated=True)

    @caption.setter
    def caption(self, value):
        self._caption = value

    @property
    def description(self):
        return self.get_attr('_description', '', translated=True)

    @description.setter
    def description(self, value):
        self._description = value

    @property
    def category(self):
        return self.get_attr('_category')

    @category.setter
    def category(self, value):
        self._category = value

    @property
    def brand(self):
        return self.get_attr('_brand')

    @brand.setter
    def brand(self, value):
        self._brand = value

    @property
    def manufacturer(self):
        return self.get_attr('_manufacturer')

    @manufacturer.setter
    def manufacturer(self, value):
        self._manufacturer = value

    @property
    def unit_price(self):
        return self.get_attr('_unit_price', 0)

    @unit_price.setter
    def unit_price(self, value):
        self._unit_price = value

    @property
    def discount(self):
        return self.get_attr('_discount')

    @discount.setter
    def discount(self, value):
        self._discount = value

    @property
    def tax(self):
        return self.get_attr('_tax') or getattr(self.category, 'tax', None)

    @tax.setter
    def tax(self, value):
        self._tax = value

    @property
    def width(self):
        return self.get_attr('_width')

    @width.setter
    def width(self, value):
        self._width = value if isinstance(value, Distance) else Distance(m=value)

    @property
    def height(self):
        return self.get_attr('_height')

    @height.setter
    def height(self, value):
        self._height = value if isinstance(value, Distance) else Distance(m=value)

    @property
    def depth(self):
        return self.get_attr('_depth')

    @depth.setter
    def depth(self, value):
        self._depth = value if isinstance(value, Distance) else Distance(m=value)

    @property
    def weight(self):
        return self.get_attr('_weight')

    @weight.setter
    def weight(self, value):
        self._weight = value if isinstance(value, Mass) else Mass(g=value)

    @property
    def price(self):
        return self.get_price()

    @property
    def is_discounted(self):
        return bool(self.discount_percent)

    @property
    def is_taxed(self):
        return bool(self.tax_percent)

    @property
    def discount_percent(self):
        return self.discount or Decimal('0.00')

    @property
    def tax_percent(self):
        return getattr(self.tax, 'percent', Decimal('0.00'))

    @property
    def discount_amount(self):
        return self.unit_price * self.discount_percent / 100

    @property
    def tax_amount(self):
        return (self.unit_price - self.discount_amount) * self.tax_percent / 100

    @property
    def primary_image(self):
        return self.images.first()

    @cached_property
    def images(self):
        images = self.attachments.filter(kind=Attachment.IMAGE)
        return images or (self.group.images if self.is_variant else images)

    @cached_property
    def videos(self):
        videos = self.attachments.filter(kind=Attachment.VIDEO)
        return videos or (self.group.videos if self.is_variant else videos)

    @cached_property
    def files(self):
        files = self.attachments.filter(kind=Attachment.FILE)
        return files or (self.group.files if self.is_variant else files)

    def get_price(self, request=None):
        """
        Returns price with discount and tax calculated.
        """
        return self.unit_price - self.discount_amount + self.tax_amount

    def get_availability(self, request=None):
        """
        Returns product availibility as list of tuples `(quantity, until)`
        Method is not yet implemented in django-shop.
        """
        availability = self.quantity if self.quantity is not None else True
        if self.is_group:
            availability = 0
        elif self.is_variant and self not in self.group.get_variants():
            availability = 0
        return [(availability, datetime.max)]

    def is_available(self, quantity=1, request=None):
        """
        Returns if product is available for the given quantity. If request
        is passed in, count items already in cart.
        """
        if request:
            cart = Cart.objects.get_or_create_from_request(request)
            cart_item = self.is_in_cart(cart)
            quantity += cart_item.quantity if cart_item else 0

        now = timezone.now().replace(tzinfo=None)
        number, until = self.get_availability(request)[0]
        number = 100000 if number is True else number
        available = number >= quantity and now < until
        return available, int(number - quantity)

    def get_modifiers(self, distinct=True):
        """
        Returns all modifiers for this product.
        Collects categorization and group modifiers.
        """
        mods = getattr(self, '_mods', None)
        if mods is None:
            mods = self.modifiers.active()
            if self.is_variant:
                mods = mods | self.group.get_modifiers(distinct=False)
            else:
                if self.category:
                    mods = mods | self.category.get_modifiers(distinct=False)
                if self.brand:
                    mods = mods | self.brand.get_modifiers(distinct=False)
                if self.manufacturer:
                    mods = mods | self.manufacturer.get_modifiers(distinct=False)
            self.cache('_mods', mods)
        return mods.distinct() if distinct else mods

    def get_flags(self, distinct=True):
        """
        Returns all flags for this product.
        Collects categorization and group flags.
        """
        flags = getattr(self, '_flags', None)
        if flags is None:
            flags = self.flags.active()
            if self.is_variant:
                flags = flags | self.group.get_flags(distinct=False)
            if self.category:
                flags = flags | self.category.get_flags(distinct=False)
            if self.brand:
                flags = flags | self.brand.get_flags(distinct=False)
            if self.manufacturer:
                flags = flags | self.manufacturer.get_flags(distinct=False)
            self.cache('_flags', flags)
        return flags.distinct() if distinct else flags

    def get_available_attributes(self):
        """
        Returns list of available attributes for Group and
        Variant products.
        """
        if self.is_group:
            if not hasattr(self, '_available_attributes'):
                self.cache('_available_attributes', self.available_attributes.active())
            return getattr(self, '_available_attributes')

    def get_attributes(self):
        """
        Returns a dictionary containing Variant attributes.
        """
        if self.is_variant:
            attrs = getattr(self, '_attributes', OrderedDict())
            if not attrs:
                for value in self.attribute_values.select_related('attribute'):
                    attrs[value.attribute.key] = value.as_dict
                self.cache('_attributes', attrs)
            return attrs

    def get_variants(self):
        """
        Returns valid variants of a Group product.
        """
        if self.is_group:
            variants = getattr(self, '_variants', None)
            if variants is None:
                variants = self.variants.all()
                invalid = [x.pk for x in self.get_invalid_variants()]
                variants = self.variants.exclude(pk__in=invalid)
                self.cache('_variants', variants)
            return variants

    def get_invalid_variants(self):
        """
        Returns variants that whose attributes don't match available
        attributes and they need to be re-configured or deleted.
        """
        if self.is_group:
            if not hasattr(self, '_invalid_variants'):
                invalid = []
                valid_attrs = []  # Keep track of valid attrs to check for duplicates.
                codes = sorted(self.get_available_attributes().values_list('code', flat=True))
                for variant in self.variants.all():
                    attrs = variant.get_attributes()
                    if (attrs in valid_attrs or sorted(x['code'] for x in attrs.values()) != codes or
                            True in [not x['nullable'] and x['value'] == '' for x in attrs.values()]):
                        invalid.append(variant)
                    else:
                        valid_attrs.append(attrs)
                self.cache('_invalid_variants', invalid)
            return getattr(self, '_invalid_variants')

    def get_variations(self):
        """
        Returns a list of tuples containing a variant id and it's attributes.
        """
        if self.is_group:
            if not hasattr(self, '_variations'):
                variations = [(x.pk, x.get_attributes()) for x in self.get_variants()]
                self.cache('_variations', variations)
            return getattr(self, '_variations')

    def get_attribute_choices(self):
        """
        Returns available attribute choices for a group product, filtering
        only the used ones. Used to display dropdown fields on a group
        product to select a variant.
        """
        if self.is_group:
            if not hasattr(self, '_attribute_choices'):
                used = [tuple([y['code'], y['value']]) for x in self.get_variations() for y in x[1].values()]
                attrs = OrderedDict()
                for attr in self.get_available_attributes():
                    data = attr.as_dict
                    data['choices'] = [x.as_dict for x in attr.get_choices() if (x.attribute.code, x.value) in used]
                    if data['choices']:
                        attrs[attr.code] = data
                self.cache('_attribute_choices', attrs)
            return getattr(self, '_attribute_choices')

    def get_combinations(self):
        """
        Returns all available Variant combinations for a Group product
        based on `Available attributes` field, replacing the existant
        variants with actual variant data. Variants with attributes
        missing or not specified in `Available attributes` will not be
        included. This is used to show possible combinations in admin,
        as well as creating them automatically.
        """
        if self.is_group:
            if not hasattr(self, '_combinations'):
                values = []
                for attr in self.get_available_attributes():
                    vals = [AttributeValue(attribute=attr, choice=x) for x in attr.get_choices()]
                    if vals:
                        values.append(vals)
                combinations = []
                if values:
                    for combo in itertools.product(*values):
                        attrs = OrderedDict([(x.attribute.code, x.value) for x in combo])
                        name = self.safe_translation_getter('name', any_language=True)
                        name = '%s %s' % (name, ' '.join([x.label for x in combo if x.label != '-']))
                        name = name.rstrip()
                        slug = slugify(name)
                        languages = []
                        variant = self.get_variant(attrs)
                        if variant:
                            name = variant.safe_translation_getter('name', name)
                            slug = variant.safe_translation_getter('slug', slug)
                            languages = variant.get_available_languages()
                        combinations.append({
                            'pk': variant.pk if variant else None,
                            'name': name,
                            'slug': slug,
                            'code': variant.code if variant else None,
                            'price': variant.get_price() if variant else None,
                            'quantity': variant.quantity if variant else None,
                            'languages': languages,
                            'attributes': OrderedDict([(x.attribute.code, x.as_dict) for x in combo])
                        })
                self.cache('_combinations', combinations)
            return getattr(self, '_combinations')

    def get_attachments(self):
        """
        Returns all attachments as a dictionary.
        If Product is a Variant and has not attachments itself,
        group attachemts are inherited.
        """
        attachments = getattr(self, '_attachments', None)
        if attachments is None:
            attachments = {
                'images': [x.as_dict for x in self.images] or None,
                'videos': [x.as_dict for x in self.videos] or None,
                'files': [x.as_dict for x in self.files] or None,
            }
            self.cache('_attachments', attachments)
        return attachments

    def get_relations(self):
        """
        Returns relations for a Single or Group product.
        """
        if not self.is_variant:
            if not hasattr(self, '_relations'):
                self.cache('_relations', self.relations.all())
            return getattr(self, '_relations')

    def get_related_products(self, kind=None):
        """
        Returns related products with the given kind. Variants inherit their
        related products from a Group.
        """
        if self.is_variant:
            return self.group.get_related_products(kind)
        relations = self.get_relations()
        if kind is not None:
            relations = relations.filter(kind=kind)
        return [x.product for x in relations]

    def get_reviews(self, language=None, include_inactive=False):
        """
        Returns reviews for this product, uses the group product for varaints.
        """
        if not self.is_variant:
            reviews = getattr(self, '_reviews', None)
            if include_inactive:
                reviews = self.reviews.all()
            if reviews is None:
                reviews = self.reviews.active()
                self.cache('_reviews', reviews)
            if language is not None:
                return reviews.filter(language=language)
            return reviews

    def get_variant(self, attrs):
        """
        Returns a Variant with the given attribute values for this Group.
        eg. attrs = {'code': 'value', 'code2': 'value2'}
        """
        if self.is_group:
            for variant in self.get_variants():
                current = [(x['code'], x['value']) for x in variant.get_attributes().values()]
                if (sorted(attrs.items()) == sorted(current)):
                    return variant

    def filter_variants(self, attrs):
        """
        Returns a list of Variant products for this Group that contain
        attribute values passed in as `attrs`.

        eg. attrs = {'code': 'value', 'code2': 'value2'}
        """
        if self.is_group:
            variants = []
            for variant in self.get_variants():
                valid = True
                current = [(x['code'], x['value']) for x in variant.get_attributes().values()]
                for attr in attrs.items():
                    if attr not in current:
                        valid = False
                        break
                if valid:
                    variants.append(variant)
            return variants

    def create_variant(self, combo, language=None):
        """
        Create a variant with the given `combo` object from
        the `get_combinations` method.
        """
        if self.is_group:
            if not language:
                language = get_current_language()
            slug = combo['slug']
            code = combo['code'] or Product.objects.latest('pk').pk + 1
            num = 0
            while Product.objects.translated(slug=slug).exists():
                num = num + 1
                slug = '%s-%d' % (combo['slug'], num)
            while Product.objects.filter(code=code, translations__language_code=language).exists():
                code = int(code) + 1
            variant, created = Product.objects.get_or_create(code=code, kind=Product.VARIANT, group=self)
            variant.set_current_language(language)
            variant.name = combo['name']
            variant.slug = slug
            variant.save()
            if created:
                for attr_value in combo['attributes'].values():
                    attr = Attribute.objects.get(code=attr_value['code'])
                    if attr_value['value'] == '' and attr.nullable:
                        choice = None
                    else:
                        choice = attr.choices.get(pk=attr_value['choice'])
                    AttributeValue.objects.create(attribute=attr, product=variant, choice=choice)
            return variant

    @method_decorator(transaction.atomic)
    def create_all_variants(self, language=None):
        """
        Creates all missing variants for the group.
        """
        if self.is_group:
            variants = []
            if not language:
                language = get_current_language()
            for combo in self.get_combinations():
                if not combo['pk'] or language not in combo['languages']:
                    variants.append(self.create_variant(combo, language=language))
            return variants

    def get_attr(self, name, case=None, translated=False):
        """
        Returns groups attribute for variants if case is True.
        Caches the attribute.
        """
        if not hasattr(self, '_%s' % name):
            attr = self.safe_translation_getter(name) or case if translated else getattr(self, name, case)
            if self.is_variant and attr == case:
                if translated:
                    attr = self.group.safe_translation_getter(name) or attr
                else:
                    attr = getattr(self.group, name, attr)
            self.cache('_%s' % name, attr)
        return getattr(self, '_%s' % name)

    def cache(self, key, value):
        """
        Used in place of `setattr` to cache an attribute.
        Keeps a list of cached attributes so that they can be cleared later.
        """
        cached = getattr(self, '_cached_attrs', [])
        if key not in cached:
            cached.append(key)
        setattr(self, key, value)
        setattr(self, '_cached_attrs', cached)

    def clear(self, *attrs):
        """
        Clears cached attributes for this instance. Optionally pass in
        specific attributes to clear.
        """
        cached = getattr(self, '_cached_attrs', [])
        attrs = [x for x in attrs if x in cached] if attrs else cached
        for attr in attrs:
            delattr(self, attr)
            cached.remove(attr)
        setattr(self, '_cached_attrs', cached)

    def clean(self):
        kinds = dict([(0, 'single'), (1, 'group'), (2, 'variant')])
        getattr(self, '_clean_%s' % kinds[self.kind])()

    def _clean_single(self):
        if self.group_id:
            raise ValidationError(em('group_has_group'))
        if self.variants.exists():
            raise ValidationError(em('not_group_has_variants'))

    def _clean_group(self):
        if self.group_id:
            raise ValidationError(em('group_has_group'))

    def _clean_variant(self):
        if self._category_id or self._brand_id or self._manufacturer_id:
            raise ValidationError(em('variant_has_category'))
        if self._tax_id:
            raise ValidationError(em('variant_has_tax'))
        if not self.group_id:
            raise ValidationError(em('variant_no_group'))
        if self.group.is_variant:
            raise ValidationError(em('varinat_group_variant'))
        if self.variants.exists():
            raise ValidationError(em('not_group_has_variants'))


class AttributeQuerySet(TranslatableQuerySet):
    def active(self):
        return self.filter(active=True)


@python_2_unicode_compatible
class Attribute(TranslatableModel):
    """
    Used to define different types of attributes to be assigned on a Product
    variant. Eg. For a t-shirt attributes could be size, color, pattern etc.
    """
    TEMPLATES = app_settings.ATTRIBUTE_TEMPLATES

    translations = TranslatedFields(
        name=models.CharField(
            _('Name'),
            max_length=128,
        ),
    )

    code = models.SlugField(
        _('Code'),
        max_length=128,
        unique=True,
        help_text=_("An identifier that's used to access this attribute. Must be unique."),
    )

    template = models.CharField(
        _('Template'),
        max_length=255,
        blank=True,
        null=True,
        choices=TEMPLATES,
        help_text=_(
            'You can specify a template for rendering this attribute or leave it empty for the '
            'default (dropdown) look.'
        ),
    )

    nullable = models.BooleanField(
        _('Nullable'),
        default=False,
        help_text=_('Check this if you want to make "empty" an option for this Attribute.'),
    )

    active = models.BooleanField(
        _('Active'),
        default=True,
        help_text=_('Is this attribute publicly visible.'),
    )

    created_at = models.DateTimeField(
        _('Created at'),
        auto_now_add=True,
    )

    updated_at = models.DateTimeField(
        _('Updated at'),
        auto_now=True,
    )

    order = models.PositiveIntegerField(
        _('Sort'),
        default=0,
    )

    objects = AttributeQuerySet.as_manager()

    class Meta:
        db_table = 'shopit_attributes'
        verbose_name = _('Attribute')
        verbose_name_plural = _('Attributes')
        ordering = ['order']

    def __str__(self):
        name = self.safe_translation_getter('name', any_language=True)
        if Attribute.objects.translated(name=name).count() > 1:
            return '%s (%s)' % (name, self.code)
        return name

    def get_choices(self):
        """
        Returns choices adding in an empty one if attribute is nullable.
        """
        choices = getattr(self, '_choices', None)
        if choices is None:
            choices = list(self.choices.all())
            if self.nullable:
                choices.insert(0, AttributeChoice(attribute=self, value=''))
            setattr(self, '_choices', choices)
        return choices

    @property
    def key(self):
        return self.code.replace('-', '_')

    @cached_property
    def as_dict(self):
        return {
            'name': self.safe_translation_getter('name', any_language=True),
            'code': self.code,
            'template': self.template,
            'nullable': self.nullable,
            'order': self.order,
        }


@python_2_unicode_compatible
class AttributeChoice(TranslatableModel):
    """
    Choices of a particular attribute.
    """
    attribute = models.ForeignKey(
        Attribute,
        models.CASCADE,
        related_name='choices',
        verbose_name=_('Attribute'),
    )

    translations = TranslatedFields(
        name=models.CharField(
            _('Name'),
            max_length=128,
            blank=True,
        ),
    )

    value = models.CharField(
        _('Value'),
        max_length=255,
    )

    file = FilerFileField(
        blank=True,
        null=True,
        verbose_name=_('File'),
    )

    order = models.PositiveIntegerField(
        _('Sort'),
        default=0,
    )

    class Meta:
        db_table = 'shopit_attribute_choices'
        verbose_name = _('Choice')
        verbose_name_plural = _('Choices')
        unique_together = [('attribute', 'value')]
        ordering = ['order']

    def __str__(self):
        return self.safe_translation_getter('name') or self.value or '-'

    @cached_property
    def as_dict(self):
        return {
            'name': str(self),
            'value': self.value,
            'file': self.file.url if self.file else None,
            'order': self.order,
        }


@python_2_unicode_compatible
class AttributeValue(models.Model):
    """
    Through model for Product attributes.
    """
    attribute = models.ForeignKey(
        Attribute,
        models.CASCADE,
        related_name='values',
        verbose_name=_('Attribute'),
    )

    product = models.ForeignKey(
        Product,
        models.CASCADE,
        related_name='attribute_values',
        verbose_name=_('Product'),
    )

    choice = models.ForeignKey(
        AttributeChoice,
        models.CASCADE,
        blank=True,
        null=True,
        verbose_name=_('Value'),
    )

    class Meta:
        db_table = 'shopit_attribute_values'
        verbose_name = _('Attribute Value')
        verbose_name_plural = _('Attribute Values')
        unique_together = ['attribute', 'product']
        ordering = ['attribute__order']

    def __str__(self):
        return self.label or '-'

    @property
    def label(self):
        return str(self.choice) if self.choice else '-'

    @property
    def value(self):
        return self.choice.value if self.choice else ''

    @property
    def file(self):
        return self.choice.file.url if self.choice and self.choice.file else None

    @cached_property
    def as_dict(self):
        data = {
            'choice': self.choice_id,
            'label': self.label,
            'value': self.value,
            'file': self.file,
        }
        data.update(self.attribute.as_dict)
        return data


@python_2_unicode_compatible
class Attachment(models.Model):
    """
    Product attachment model.
    """
    IMAGE, VIDEO, FILE = ('image', 'video', 'file')

    KINDS = (
        (IMAGE, _('Image')),
        (VIDEO, _('Video')),
        (FILE, _('File')),
    )

    EXTENSIONS = {
        IMAGE: ['jpg', 'jpeg', 'png', 'gif'],
        VIDEO: ['webm', 'ogv', 'ogg', 'mp4', 'm4p', 'm4v', 'flv', 'f4v'],
        FILE: ['pdf', 'doc', 'docx', 'ppt', 'pptx', 'txt'],
    }

    product = models.ForeignKey(
        Product,
        models.CASCADE,
        related_name='attachments',
        verbose_name=_('Product'),
    )

    kind = models.CharField(
        _('Kind'),
        max_length=16,
        choices=KINDS,
        default=IMAGE,
    )

    file = FilerFileField(
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        verbose_name=_('File'),
    )

    url = models.URLField(
        _('Url'),
        blank=True,
    )

    order = models.PositiveIntegerField(
        _('Sort'),
        default=0,
    )

    class Meta:
        db_table = 'shopit_attachments'
        verbose_name = _('Attachment')
        verbose_name_plural = _('Attachments')
        ordering = ['order']

    def __str__(self):
        return self.label

    def save(self, *args, **kwargs):
        self.clean()
        super(Attachment, self).save(*args, **kwargs)

    @property
    def label(self):
        return self.file.label if self.file else basename(self.value)

    @property
    def value(self):
        return self.file.url if self.file else self.url

    @cached_property
    def as_dict(self):
        attachment = {
            'label': self.label,
            'url': self.value,
            'order': self.order,
        }
        if EASY_THUMBNAILS and self.kind == self.IMAGE and self.file is not None:
            try:
                thumbnailer = get_thumbnailer(self.file)
                for alias, options in aliases.all(target='shopit.Attachment').items():
                    options.update({'subject_location': self.file.subject_location})
                    attachment['url_%s' % alias] = thumbnailer.get_thumbnail(options).url
            except InvalidImageFormatError:
                pass
        return attachment

    def clean(self):
        if self.file:
            if self.file.extension not in self.EXTENSIONS[self.kind]:
                raise ValidationError('%s (%s)' % (em('wrong_extension'), ', '.join(self.EXTENSIONS[self.kind])))
        elif not self.url:
            raise ValidationError(em('no_attachment_or_url'))


@python_2_unicode_compatible
class Relation(models.Model):
    """
    Inline product relation model.
    """
    KINDS = app_settings.RELATION_KINDS

    base = models.ForeignKey(
        Product,
        models.CASCADE,
        related_name='relations',
    )

    product = models.ForeignKey(
        Product,
        models.CASCADE,
        verbose_name=_('Product'),
        limit_choices_to={'kind__in': [Product.SINGLE, Product.GROUP]},
    )

    kind = models.CharField(
        _('Kind'),
        max_length=128,
        blank=True,
        choices=KINDS,
    )

    order = models.PositiveIntegerField(
        _('Sort'),
        default=0,
    )

    class Meta:
        db_table = 'shopit_relations'
        verbose_name = _('Relation')
        verbose_name_plural = _('Relations')
        unique_together = [('base', 'product', 'kind')]
        ordering = ['order']

    def __str__(self):
        return self.product.product_name

    def save(self, *args, **kwargs):
        self.clean()
        super(Relation, self).save(*args, **kwargs)

    def clean(self):
        if self.base.is_variant or self.product.is_variant:
            raise ValidationError(em('variant_has_relations'))
        if self.base == self.product:
            raise ValidationError(em('relation_base_is_product'))


class ReviewQuerySet(QuerySet):
    def active(self):
        return self.filter(active=True)


@python_2_unicode_compatible
class Review(models.Model):
    """
    Model for product reviews.
    """
    RATINGS = app_settings.REVIEW_RATINGS

    product = models.ForeignKey(
        Product,
        models.CASCADE,
        related_name='reviews',
        verbose_name=_('Product'),
        limit_choices_to={'kind__in': [Product.SINGLE, Product.GROUP]},
    )

    customer = models.ForeignKey(
        Customer,
        models.CASCADE,
        related_name='product_reviews',
        verbose_name=_('Customer'),
    )

    name = models.CharField(
        _('Name'),
        max_length=128,
        blank=True,
    )

    text = models.TextField(
        _('Text'),
        max_length=1024,
    )

    rating = models.PositiveIntegerField(
        _('Rating'),
        choices=RATINGS,
        default=0,
    )

    language = models.CharField(
        _('Language'),
        max_length=2,
        choices=settings.LANGUAGES,
        default=settings.LANGUAGES[0][0],
    )

    active = models.BooleanField(
        _('Active'),
        default=True,
        help_text=_('Is this review publicly visible.'),
    )

    created_at = models.DateTimeField(
        _('Created at'),
        auto_now_add=True,
    )

    updated_at = models.DateTimeField(
        _('Updated at'),
        auto_now=True,
    )

    order = models.PositiveIntegerField(
        _('Sort'),
        default=0,
    )

    objects = ReviewQuerySet.as_manager()

    class Meta:
        db_table = 'shopit_reviews'
        verbose_name = _('Review')
        verbose_name_plural = _('Reviews')
        ordering = ['-order']

    def __str__(self):
        return truncatewords(self.text, 3)

    def get_absolute_url(self):
        try:
            return reverse(
                'shopit-product-review-detail',
                args=[self.product.safe_translation_getter('slug'), self.id]
            )
        except NoReverseMatch:  # pragma: no cover
            pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from cms.cms_toolbars import ADMIN_MENU_IDENTIFIER, ADMINISTRATION_BREAK
from cms.toolbar.items import Break, SubMenu
from cms.toolbar_base import CMSToolbar
from cms.toolbar_pool import toolbar_pool
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from shopit.models.categorization import Brand, Category, Manufacturer
from shopit.models.product import Product


@toolbar_pool.register
class ShopitToolbar(CMSToolbar):
    """
    djangoCMS toolbar for Shopit.
    """
    watch_models = [Product, Category, Brand, Manufacturer]

    def populate(self):
        admin_menu = self.toolbar.get_menu(ADMIN_MENU_IDENTIFIER)
        if admin_menu:  # pragma: no cover
            position = admin_menu.get_alphabetical_insert_position(_('Shopit'), SubMenu)
            if not position:
                position = admin_menu.find_first(Break, identifier=ADMINISTRATION_BREAK) + 1
                admin_menu.add_break('shopit-break', position=position)

            menu = admin_menu.get_or_create_menu('shopit-admin-menu', _('Shopit'), position=position)

            product_changelist_url = reverse('admin:shopit_product_changelist')
            menu.add_sideframe_item(_('Product List'), url=product_changelist_url)
            product_add_url = reverse('admin:shopit_product_add')
            menu.add_modal_item(_('Add New Product'), url=product_add_url)
            menu.add_break()

            category_changelist_url = reverse('admin:shopit_category_changelist')
            menu.add_sideframe_item(_('Categories List'), url=category_changelist_url)
            category_add_url = reverse('admin:shopit_category_add')
            menu.add_modal_item(_('Add New Category'), url=category_add_url)
            menu.add_break()

            brand_changelist_url = reverse('admin:shopit_brand_changelist')
            menu.add_sideframe_item(_('Brands List'), url=brand_changelist_url)
            brand_add_url = reverse('admin:shopit_brand_add')
            menu.add_modal_item(_('Add New Brand'), url=brand_add_url)
            menu.add_break()

            manufacturer_changelist_url = reverse('admin:shopit_manufacturer_changelist')
            menu.add_sideframe_item(_('Manufacturers List'), url=manufacturer_changelist_url)
            manufacturer_add_url = reverse('admin:shopit_manufacturer_add')
            menu.add_modal_item(_('Add New Manufacturer'), url=manufacturer_add_url)

        if self.is_current_app:
            menu = self.toolbar.get_or_create_menu('shopit-menu', _('Shopit'))
            add_menu = menu.get_or_create_menu('shopit-menu-add', _('Add New'))

            add_menu.add_modal_item(_('Product'), url=product_add_url)
            add_menu.add_modal_item(_('Category'), url=category_add_url)
            add_menu.add_modal_item(_('Brand'), url=brand_add_url)
            add_menu.add_modal_item(_('Manufacturer'), url=manufacturer_add_url)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import copy

from cms.admin.placeholderadmin import FrontendEditableAdminMixin, PlaceholderAdminMixin
from django.contrib import admin
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _
from mptt.admin import DraggableMPTTAdmin
from parler.admin import TranslatableAdmin

from shopit.forms.categorization import BrandModelForm, CategoryModelForm, ManufacturerModelForm
from shopit.models.categorization import Brand, Category, Manufacturer


class CategorizationAdminBase(FrontendEditableAdminMixin, PlaceholderAdminMixin,
                              TranslatableAdmin, DraggableMPTTAdmin):
    """
    Base admin for categorization models.
    """
    list_display = ['tree_actions', 'get_name', 'slug', 'active', 'language_column']
    list_display_links = ['get_name']
    filter_horizontal = ['modifiers', 'flags']
    readonly_fields = ['created_at', 'updated_at']

    frontend_editable_fields = [
        'name', 'slug', 'active', 'created_at', 'updated_at', 'description', '_featured_image', 'parent',
        'modifiers', 'flags']

    fieldsets = [
        (_('Basic info'), {'fields': ['name', 'slug']}),
        (_('Status'), {'fields': [('active', 'created_at', 'updated_at')]}),
        (_('Description'), {'fields': ['description']}),
        (_('Media'), {'fields': ['_featured_image']}),
        (_('Settings'), {'fields': ['parent', 'modifiers', 'flags']}),
    ]

    class Media:
        css = {'all': ['shopit/css/djangocms-admin-style.css']}

    def get_prepopulated_fields(self, request, obj=None):
        return {'slug': ['name']}  # pragma: no cover

    def get_name(self, obj):
        return format_html(
            '<div style="text-indent:{}px">{}</div>',
            obj.level * self.mptt_level_indent, obj.safe_translation_getter('name', any_language=True))
    get_name.short_description = _('Name')


@admin.register(Category)
class CategoryAdmin(CategorizationAdminBase):
    form = CategoryModelForm
    frontend_editable_fields = CategorizationAdminBase.frontend_editable_fields + ['_tax']

    def get_list_display(self, request):
        list_display = list(super(CategoryAdmin, self).get_list_display(request))
        list_display.insert(-2, '_tax')
        return list_display

    def get_fieldsets(self, request, obj=None):
        fieldsets = copy.deepcopy(super(CategoryAdmin, self).get_fieldsets(request, obj))
        fieldsets[-1][1]['fields'].append('_tax')
        return fieldsets


@admin.register(Brand)
class BrandAdmin(CategorizationAdminBase):
    form = BrandModelForm


@admin.register(Manufacturer)
class ManufacturerAdmin(CategorizationAdminBase):
    form = ManufacturerModelForm



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.http import Http404
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _
from parler.views import ViewUrlMixin
from rest_framework.generics import ListAPIView
from rest_framework.settings import api_settings

from shopit.models.categorization import Brand, Category, Manufacturer
from shopit.rest.renderers import ModifiedCMSPageRenderer
from shopit.serializers import BrandSerializer, CategorySerializer, ManufacturerSerializer
from shopit.views.product import ProductListView


class CategorizationViewMixin(object):
    """
    Categorization mixin that makes sure `categorization_model` is specified
    and generates `categorization_model_name`.
    """
    categorization_model = None
    categorization_model_name = None

    def __init__(self, *args, **kwargs):
        super(CategorizationViewMixin, self).__init__(*args, **kwargs)
        if self.categorization_model is None:
            raise ImproperlyConfigured('Must specify `categorization_model` attribute.')

        if self.categorization_model_name is None:
            self.categorization_model_name = self.categorization_model._meta.model.__name__.lower()


class CategorizationListViewBase(CategorizationViewMixin, ListAPIView):
    """
    Base categorization list view.
    """
    serializers_class = None
    renderer_classes = [ModifiedCMSPageRenderer] + api_settings.DEFAULT_RENDERER_CLASSES
    template_name = 'shopit/catalog/categorization_list.html'

    def get_queryset(self):
        return self.categorization_model.objects.active()

    def get_template_names(self):
        return ['shopit/catalog/%s_list.html' % self.categorization_model_name, self.template_name]

    def get_renderer_context(self):
        context = super(CategorizationListViewBase, self).get_renderer_context()
        if context['request'].accepted_renderer.format == 'html':
            queryset = self.filter_queryset(self.get_queryset())
            page = self.paginate_queryset(queryset)
            if page is not None:
                context.update(self.paginator.get_html_context())
            context['categorization_list'] = context['%s_list' % self.categorization_model_name] = page or queryset
        return context


class CategoryListView(CategorizationListViewBase):
    categorization_model = Category
    serializer_class = CategorySerializer


class BrandListView(CategorizationListViewBase):
    categorization_model = Brand
    serializer_class = BrandSerializer


class ManufacturerListView(CategorizationListViewBase):
    categorization_model = Manufacturer
    serializer_class = ManufacturerSerializer


class CategorizationDetailViewBase(CategorizationViewMixin, ViewUrlMixin, ProductListView):
    """
    Base categorization detail view.
    A ProductListView with filtered products by current categorization layer.
    """
    template_name = 'shopit/catalog/categorization_detail.html'

    def get(self, request, *args, **kwargs):
        response = super(CategorizationDetailViewBase, self).get(request, *args, **kwargs)
        categorization_id = self.get_categorization_object().pk
        name = self.categorization_model_name
        change_url = reverse('admin:shopit_%s_change' % name, args=[categorization_id])
        delete_url = reverse('admin:shopit_%s_delete' % name, args=[categorization_id])
        menu = request.toolbar.get_or_create_menu('shopit-menu', _('Shopit'))
        menu.add_break()
        menu.add_modal_item(_('Edit %s') % name.capitalize(), url=change_url)
        menu.add_sideframe_item(_('Delete %s') % name.capitalize(), url=delete_url)
        return response

    def get_queryset(self):
        queryset = super(CategorizationDetailViewBase, self).get_queryset()
        categorization = self.get_categorization_object()
        ids = categorization.get_descendants(include_self=True).values_list('pk', flat=True)
        return queryset.filter(**{'_%s_id__in' % self.categorization_model_name: ids})

    def get_template_names(self):
        return ['shopit/catalog/%s_detail.html' % self.categorization_model_name, self.template_name]

    def get_renderer_context(self):
        context = super(CategorizationDetailViewBase, self).get_renderer_context()
        context['categorization'] = context[self.categorization_model_name] = self.get_categorization_object()
        return context

    def get_categorization_object(self):
        if not hasattr(self, '_categorization_object'):
            path = self.kwargs['path']
            slug = path.split('/').pop()
            queryset = self.categorization_model.objects.translated(slug=slug)
            categorization = get_object_or_404(queryset)
            if categorization.get_path() != path:
                raise Http404
            setattr(self, '_categorization_object', categorization)
        return getattr(self, '_categorization_object')

    def get_view_url(self):
        """
        Return object view url. Used in `get_translated_url` templatetag from parler.
        """
        return self.get_categorization_object().get_absolute_url()


class CategoryDetailView(CategorizationDetailViewBase):
    categorization_model = Category


class BrandDetailView(CategorizationDetailViewBase):
    categorization_model = Brand


class ManufacturerDetailView(CategorizationDetailViewBase):
    categorization_model = Manufacturer



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.core.urlresolvers import NoReverseMatch, reverse
from django.template.loader import select_template
from django.utils import six
from measurement.base import MeasureBase
from measurement.measures import Distance, Mass
from rest_auth.serializers import PasswordResetConfirmSerializer, PasswordResetSerializer
from rest_framework import serializers
from shop.rest.money import MoneyField
from shop.serializers.bases import ProductSerializer as BaseProductSerializer
from shop.serializers.cart import CartItemSerializer as BaseCartItemSerializer
from shop.serializers.cart import ExtraCartRow as BaseExtraCartRow
from shop.serializers.cart import WatchItemSerializer as BaseWatchItemSerializer
from shop.serializers.defaults import AddToCartSerializer as BaseAddToCartSerializer
from shop.serializers.defaults import CustomerSerializer
from shop.serializers.order import OrderListSerializer as BaseOrderListSerializer

from shopit.conf import app_settings
from shopit.models.address import BillingAddress, ShippingAddress
from shopit.models.categorization import Brand, Category, Manufacturer
from shopit.models.customer import Customer
from shopit.models.flag import Flag
from shopit.models.modifier import Modifier
from shopit.models.product import Product, Relation, Review
from shopit.models.tax import Tax


class AccountSerializer(CustomerSerializer):
    id = serializers.IntegerField(source='pk')
    username = serializers.CharField(source='get_username')
    shipping_addresses = serializers.SerializerMethodField()
    billing_addresses = serializers.SerializerMethodField()

    class Meta:
        model = Customer
        fields = [
            'id', 'username', 'salutation', 'first_name', 'last_name', 'email', 'phone_number', 'extra',
            'shipping_addresses', 'billing_addresses']

    def get_shipping_addresses(self, obj):
        addresses = obj.shippingaddress_set.all()
        if addresses:
            return ShippingAddressSerializer(addresses, context=self.context, many=True).data

    def get_billing_addresses(self, obj):
        addresses = obj.billingaddress_set.all()
        if addresses:
            return BillingAddressSerializer(addresses, context=self.context, many=True).data


class AccountSummarySerializer(AccountSerializer):
    def get_fields(self):
        fields = super(AccountSummarySerializer, self).get_fields()
        del fields['username']
        del fields['email']
        del fields['phone_number']
        del fields['extra']
        return fields


class AccountResetSerializer(PasswordResetSerializer):
    def get_email_options(self):
        subject_template = select_template(['shopit/email/account_reset_subject.txt'])
        body_template = select_template(['shopit/email/account_reset_body.txt'])
        return {
            'subject_template_name': subject_template.template.name,
            'email_template_name': body_template.template.name
        }


class AccountResetConfirmSerializer(PasswordResetConfirmSerializer):
    pass


class AddressSerializerBase(serializers.ModelSerializer):
    class Meta:
        exclude = ['customer']


class OrderListSerializer(BaseOrderListSerializer):
    url = serializers.SerializerMethodField()

    def get_url(self, obj):
        url = obj.get_absolute_url()
        return self.context['request'].build_absolute_uri(url) if url else None


class ShippingAddressSerializer(AddressSerializerBase):
    class Meta(AddressSerializerBase.Meta):
        model = ShippingAddress


class BillingAddressSerializer(AddressSerializerBase):
    class Meta(AddressSerializerBase.Meta):
        model = BillingAddress


class TaxSerializer(serializers.ModelSerializer):
    class Meta:
        model = Tax
        fields = ['id', 'name', 'percent']


class FlagSerializer(serializers.ModelSerializer):
    path = serializers.SerializerMethodField(read_only=True)

    class Meta:
        model = Flag
        fields = ['id', 'name', 'code', 'template', 'path']

    def get_path(self, obj):
        return '/'.join(obj.get_ancestors(include_self=True).values_list('code', flat=True))


class ModifierSerializer(serializers.ModelSerializer):
    amount = MoneyField()

    class Meta:
        model = Modifier
        fields = ['id', 'name', 'code', 'amount', 'percent', 'kind', 'order']


class CategorizationSerializerBase(serializers.ModelSerializer):
    """
    Base categorization serializer.
    Model and fields must be specified when extending.
    """
    url = serializers.SerializerMethodField()
    modifiers = ModifierSerializer(source='get_modifiers', many=True)
    flags = FlagSerializer(source='get_flags', many=True)

    class Meta:
        fields = ['id', 'name', 'slug', 'url', 'parent', 'modifiers', 'flags']

    def get_url(self, obj):
        url = obj.get_absolute_url()
        return self.context['request'].build_absolute_uri(url) if url else None


class CategorySerializer(CategorizationSerializerBase):
    tax = TaxSerializer()

    class Meta(CategorizationSerializerBase.Meta):
        model = Category
        fields = CategorizationSerializerBase.Meta.fields + ['tax']


class BrandSerializer(CategorizationSerializerBase):
    class Meta(CategorizationSerializerBase.Meta):
        model = Brand


class ManufacturerSerializer(CategorizationSerializerBase):
    class Meta(CategorizationSerializerBase.Meta):
        model = Manufacturer


class MeasureField(serializers.Field):
    """
    Custom serializer field to handle Measure objects.
    """
    def __init__(self, *args, **kwargs):
        assert 'measure' in kwargs, 'Missing `measure` argument.'
        self.measure = kwargs.pop('measure')
        assert not isinstance(self.measure, MeasureBase), 'Incorrect measure class.'
        super(MeasureField, self).__init__(*args, **kwargs)

    def to_representation(self, obj):
        return str(obj)

    def to_internal_value(self, data):
        data = data.split(' ')
        if not isinstance(data, six.text_type) and len(data) != 2:
            raise serializers.ValidationError('Got unexpected data.')
        return self.measure(**{data[1]: float(data[0])})


class RelationSerializer(serializers.ModelSerializer):
    class Meta:
        model = Relation
        exclude = []


class ReviewSerializer(serializers.ModelSerializer):
    url = serializers.SerializerMethodField()

    class Meta:
        model = Review
        fields = ['id', 'customer', 'name', 'text', 'rating', 'language', 'url', 'active', 'created_at', 'updated_at']

    def get_url(self, obj):
        url = obj.get_absolute_url()
        return self.context['request'].build_absolute_uri(url) if url else None


class ProductSerializer(BaseProductSerializer):
    """
    Base product serializer.
    """
    FIELDS = app_settings.PRODUCT_SERIALIZER_FIELDS

    url = serializers.SerializerMethodField(read_only=True)
    add_to_cart_url = serializers.SerializerMethodField(read_only=True)
    unit_price = MoneyField()
    tax = TaxSerializer()
    is_available = serializers.SerializerMethodField(read_only=True)
    category = CategorySerializer()
    brand = BrandSerializer()
    manufacturer = ManufacturerSerializer()
    modifiers = ModifierSerializer(source='get_modifiers', many=True)
    flags = FlagSerializer(source='get_flags', many=True)
    width = MeasureField(measure=Distance)
    height = MeasureField(measure=Distance)
    depth = MeasureField(measure=Distance)
    weight = MeasureField(measure=Mass)
    attributes = serializers.DictField(source='get_attributes', read_only=True)
    attribute_choices = serializers.DictField(source='get_attribute_choices', read_only=True)
    discount_amount = MoneyField(read_only=True)
    tax_amount = MoneyField(read_only=True)
    variants = serializers.SerializerMethodField()
    variations = serializers.ListField(source='get_variations', read_only=True)
    attachments = serializers.SerializerMethodField()
    relations = RelationSerializer(source='get_relations', many=True)
    reviews = serializers.SerializerMethodField()

    class Meta:
        model = Product
        fields = [
            'id', 'name', 'slug', 'caption', 'code', 'kind', 'url', 'add_to_cart_url', 'price', 'is_available',
            'description', 'unit_price', 'discount', 'tax', 'availability', 'category', 'brand', 'manufacturer',
            'discountable', 'modifiers', 'flags', 'width', 'height', 'depth', 'weight', 'available_attributes',
            'group',  'attributes', 'attribute_choices', 'published', 'quantity', 'order', 'active', 'created_at',
            'updated_at', 'is_single', 'is_group', 'is_variant', 'is_discounted', 'is_taxed',  'discount_percent',
            'tax_percent', 'discount_amount', 'tax_amount', 'variants', 'variations', 'attachments', 'relations',
            'reviews',
        ]

    def get_fields(self):
        fields = super(ProductSerializer, self).get_fields()
        overriden = self.context['request'].GET.get('fields', None)
        names = [x for x in overriden.split(',') if x in self.Meta.fields] if overriden is not None else self.FIELDS
        names = list(set(names + self.get_included_fields()))
        for excluded in [x for x in fields if x not in names]:
            del fields[excluded]
        return fields

    def get_included_fields(self):
        return [x for x in self.context['request'].GET.get('include', '').split(',') if x in self.Meta.fields]

    def get_url(self, obj):
        url = obj.get_absolute_url()
        return self.context['request'].build_absolute_uri(url) if url else None

    def get_add_to_cart_url(self, obj):
        try:
            url = reverse('shopit-add-to-cart', args=[obj.safe_translation_getter('slug', any_language=True)])
            return self.context['request'].build_absolute_uri(url)
        except NoReverseMatch:
            return None

    def get_is_available(self, obj):
        return obj.is_available(request=self.context['request'])

    def get_variants(self, obj):
        variants = obj.get_variants()
        if variants:
            return ProductDetailSerializer(variants, context=self.context, many=True).data

    def get_attachments(self, obj):
        request = self.context['request']
        attachments = obj.get_attachments()
        for kind, items in [x for x in attachments.items() if x[1]]:
            for item in items:
                for key, value in item.items():
                    if key.startswith('url'):
                        item[key] = request.build_absolute_uri(value)
        return attachments

    def get_reviews(self, obj):
        reviews = obj.get_reviews(language=self.context['request'].LANGUAGE_CODE)
        if reviews:
            return ReviewSerializer(reviews, context=self.context, many=True).data


class ProductSummarySerializer(ProductSerializer):
    """
    Product list serializer is a Product serializer without some fields.
    """
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('label', 'catalog')
        super(ProductSummarySerializer, self).__init__(*args, **kwargs)


class ProductDetailSerializer(ProductSerializer):
    FIELDS = app_settings.PRODUCT_DETAIL_SERIALIZER_FIELDS

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('label', 'catalog')
        super(ProductDetailSerializer, self).__init__(*args, **kwargs)


class AddToCartSerializer(BaseAddToCartSerializer):
    is_available = serializers.ListField(read_only=True)

    def get_instance(self, context, data, extra_args):
        instance = super(AddToCartSerializer, self).get_instance(context, data, extra_args)
        product = context['product']
        request = context['request']
        quantity = int(request.GET.get('quantity', 1))
        instance.update({
            'quantity': quantity,
            'is_available': product.is_available(quantity, request),
        })
        return instance


class CartItemSerializer(BaseCartItemSerializer):
    pass


class WatchItemSerializer(BaseWatchItemSerializer):
    pass


class ExtraCartRow(BaseExtraCartRow):
    code = serializers.CharField(read_only=True, help_text="A code that's applied to the cart")



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from shop.modifiers.base import BaseCartModifier
from shop.modifiers.defaults import PayInAdvanceModifier as PayInAdvanceModifierBase
from shop.money import Money

from shopit.models.modifier import Modifier
from shopit.payment import ForwardFundPayment
from shopit.serializers import ExtraCartRow


class ShopitCartModifier(BaseCartModifier):
    """
    Applies all cart and product modifiers.
    """
    cart_discount_codes = []

    def pre_process_cart(self, cart, request):
        self.cart_discount_codes = cart.get_discount_codes().order_by('-id').values_list('code', flat=True)

    def add_extra_cart_item_row(self, cart_item, request):
        for modifier in cart_item.product.get_modifiers():
            if modifier.can_be_applied(request, cart_item=cart_item):
                amount = modifier.get_added_amount(cart_item.line_total, cart_item.quantity)
                instance = {'label': modifier.label, 'amount': amount, 'code': self.get_applied_code(modifier)}
                cart_item.extra_rows[modifier.code] = ExtraCartRow(instance)
                cart_item.line_total += amount
                if cart_item.line_total < 0:
                    cart_item.line_total = Money(0)

    def add_extra_cart_row(self, cart, request):
        for modifier in Modifier.get_cart_modifiers():
            if modifier.can_be_applied(request, cart=cart):
                amount = modifier.get_added_amount(cart.total)
                instance = {'label': modifier.label, 'amount': amount, 'code': self.get_applied_code(modifier)}
                cart.extra_rows[modifier.code] = ExtraCartRow(instance)
                cart.total += amount
                if cart.total < 0:
                    cart.total = Money(0)

    def get_applied_code(self, modifier):
        """
        Returns first code applied to the cart, that's also a valid code
        for the given modifier. Returns `None` if modifier doesn't need a code.
        """
        discount_codes = [x.code for x in modifier.get_discount_codes(include_added=True)]
        for code in self.cart_discount_codes:
            if code in discount_codes:
                return code


class PayInAdvanceModifier(PayInAdvanceModifierBase):
    """
    Modifier that makes use of a customized ForwardFundPayment.
    """
    payment_provider = ForwardFundPayment()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from shop.rest.renderers import CMSPageRenderer


class ModifiedCMSPageRenderer(CMSPageRenderer):
    """
    Modified CMSPageRender to support rendering non CMS templates.
    """
    def render(self, data, accepted_media_type=None, renderer_context=None):
        renderer_context = renderer_context or {}
        view = renderer_context['view']
        request = renderer_context['request']
        response = renderer_context['response']
        template_context = self.get_template_context(dict(data), renderer_context)

        if response.exception:
            template = self.get_exception_template(response)
        else:
            template_names = self.get_template_names(response, view)
            template = self.resolve_template(template_names)
            template_context['paginator'] = view.paginator
            if request.current_page:
                # set edit_mode, so that otherwise invisible placeholders can be edited inline
                template_context['edit_mode'] = request.current_page.publisher_is_draft

        template_context['data'] = data
        template_context.update(renderer_context)
        return template.render(template_context, request=request)
